#!/usr/bin/env python

"""
A fixation cross stimulus.

This module contains a class implementing a fixation cross stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import *


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from . import defaults
from ._shape import Shape
from .. import _internals
from ..misc.geometry import vertices_cross

class FixCross(Shape):
    """A class implementing a general fixation cross."""

    def __init__(self, size=None, position=None, line_width=None,
                 colour=None, anti_aliasing=None, cross_size=None):
        """Create a fixation cross.

        Parameters
        ----------
        size : (int, int), optional
            size of the cross
        position : (int, int), optional
            position of the stimulus
        line_width : int optional
            width of the lines
        colour : (int, int, int), optional
            colour of the cross
        anti_aliasing :  int, optional
            anti aliasing parameter (good anti_aliasing with 10)


        Notes
        -----
        The parameter cross_size is now OBSOLETE. Please use 'size' and
        specify x and y dimensions.

        """

        if cross_size is not None and size is None:
            raise DeprecationWarning("Property cross_size is obsolete. Please use size")

        if position is None:
            position = defaults.fixcross_position
        if colour is None:
            colour = defaults.fixcross_colour
        if colour is not None:
            self._colour = colour
        else:
            self._colour = _internals.active_exp.foreground_colour
        if anti_aliasing is None:
            anti_aliasing = defaults.fixcross_anti_aliasing
        Shape.__init__(self, position=position, line_width=0,
                         colour=colour,
                         anti_aliasing=anti_aliasing)
        if size is None:
            size = defaults.fixcross_size
        if line_width is None:
            line_width = defaults.fixcross_line_width

        self._size = size
        self.add_vertices(vertex_list=vertices_cross(size=size,
                                                     line_width=line_width))


    @property
    def size(self):
        """Getter for size."""

        return self._size

    @property
    def cross_size(self):
        """OBSOLETE property, please use size"""

        raise DeprecationWarning("Property cross_size is obsolete. Please use size")

    @property
    def line_width(self):
        """Getter for line_width."""

        return self._line_width


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        fixcross = FixCross(size=(100, 100))
        fixcross.present()
        if exp is None:
            exp_.clock.wait(1000)



#!/usr/bin/env python

"""
An ellipse stimulus.

This module contains a class implementing an ellipse stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import *


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import pygame

from . import defaults
from ._visual import Visual
from .. import _internals

class Ellipse(Visual):
    """A class implementing a basic 2D ellipse."""

    def __init__(self, radii, colour=None, line_width=None, position=None,
                 anti_aliasing=None):
        """Create an ellipse.

        Parameters
        ----------
        radii : (int, int)
            major and minor radii of the ellipse
        colour : (int, int, int), optional
            colour of the ellipse
        line_width : int, optional
            line width in pixels; 0 will result in a filled ellipse
            (as does a value < 0 or >= min(size))
        position : (int, int), optional
            position of the stimulus
        anti_aliasing : int, optional
            anti aliasing parameter (good anti_aliasing with 10)

        """

        if position is None:
            position = defaults.ellipse_position
        Visual.__init__(self, position)
        self._radii = radii
        self._size = [x * 2 for x in radii]
        if colour is None:
            colour = defaults.ellipse_colour
        if colour is not None:
            self._colour = colour
        else:
            self._colour = _internals.active_exp.foreground_colour
        if line_width is None:
            line_width = defaults.ellipse_line_width
        elif line_width < 0 or line_width >= min(self._size):
            line_width = 0
        self._line_width = line_width
        if anti_aliasing is not None:
            self._anti_aliasing = anti_aliasing
        else:
            self._anti_aliasing = defaults.ellipse_anti_aliasing

    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def radii(self):
        """Getter for radii."""
        return self._radii

    @radii.setter
    def radii(self, value):
        """Setter for radii."""

        if self.has_surface:
            raise AttributeError(Ellipse._getter_exception_message.format(
                "radii"))
        else:
            self._radii = value

    @property
    def colour(self):
        """Getter for colour."""
        return self._colour

    @colour.setter
    def colour(self, value):
        """Setter for colour."""

        if self.has_surface:
            raise AttributeError(Ellipse._getter_exception_message.format(
                "colour"))
        else:
            self._colour = value

    @property
    def line_width(self):
        """Getter for line_width."""
        return self._line_width

    @line_width.setter
    def line_width(self, value):
        """Setter for line_width."""

        if self.has_surface:
            raise AttributeError(Ellipse._getter_exception_message.format(
                "line_width"))
        else:
            self._line_width = value

    @property
    def anti_aliasing(self):
        """Getter for anti_aliasing."""

        return self._anti_aliasing

    @anti_aliasing.setter
    def anti_aliasing(self, value):
        """Setter for anti_aliasing."""

        if self.has_surface:
            raise AttributeError(Ellipse._getter_exception_message.format(
                "anti_aliasing"))
        self._anti_aliasing = value

    def _create_surface(self):
        """Create the surface of the stimulus."""

        if self._anti_aliasing > 0:
            aa_scaling = (self._anti_aliasing / 5.0) + 1
        else:
            aa_scaling = 1

        size = [(self._size[0]) * aa_scaling,
                (self._size[1]) * aa_scaling]

        if self._line_width == 0:
            surface = pygame.surface.Surface(
                size, pygame.SRCALPHA).convert_alpha()
            pygame.draw.ellipse(surface, (0, 0, 0), pygame.Rect(
                (0, 0), size))
            if aa_scaling != 1:
                surface = pygame.transform.smoothscale(surface,
                                (int(size[0] / aa_scaling),
                                 int(size[1] / aa_scaling)))
            surface.fill(self._colour, special_flags=pygame.BLEND_RGB_MAX)

        else:
            line_width = self._line_width * aa_scaling
            if line_width % 2 == 0:
                surface_size = [x + line_width for x in size]
            else:
                surface_size = [x + line_width - 1 for x in size]
            surface = pygame.surface.Surface(surface_size,
                pygame.SRCALPHA).convert_alpha()
            pygame.draw.ellipse(surface, (0, 0, 0), pygame.Rect(
                (0, 0),
                surface_size))#[x + line_width for x in size]))
            tmp = pygame.surface.Surface(
                [x - line_width for x in size])
            hole = pygame.surface.Surface(
                [x - line_width for x in size],
                pygame.SRCALPHA).convert_alpha()
            tmp.fill([0, 0, 0])
            pygame.draw.ellipse(tmp, (255, 255, 255), pygame.Rect(
                (0, 0), [x - line_width for x in size]))
            tmp.set_colorkey([255, 255, 255])
            hole.blit(tmp, (0, 0))
            surface.blit(hole, (line_width, line_width),
                         special_flags=pygame.BLEND_RGBA_MIN)
            if aa_scaling != 1:
                surface = pygame.transform.smoothscale(surface,
                                (int((size[0] + line_width) / aa_scaling),
                                 int((size[1] + line_width) / aa_scaling)))
            surface.fill(self._colour, special_flags=pygame.BLEND_RGB_MAX)

        return surface


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        ellipse = Ellipse(radii=[200, 100], anti_aliasing=10)
        ellipse.present()
        if exp is None:
            exp_.clock.wait(1000)



"""The stimuli package.

This Package contains a variety of classes implementing experimental stimuli.
See also expyriment.stimuli.extras for more stimuli.

"""

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from . import defaults
from ._audio import Audio
from ._video import Video
from ._canvas import Canvas
from ._circle import Circle
from ._rectangle import Rectangle
from ._line import Line
from ._ellipse import Ellipse
from ._shape import Shape
from ._blankscreen import BlankScreen
from ._textline import TextLine
from ._fixcross import FixCross
from ._textbox import TextBox
from ._textscreen import TextScreen
from ._picture import Picture
from ._tone import Tone

from ._obsolete import Dot, Frame



"""
A Trigger input

This module contains a class implementing a trigger input.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from types import FunctionType

from . import defaults
from .. import _internals
from ..misc import compare_codes
from .._internals import CallbackQuitEvent
from ..misc._timer import get_time
from ._keyboard import Keyboard
from  ._input_output import Input


class TriggerInput(Input):
    """A class implementing a trigger input."""

    def __init__(self, interface, default_code=None):
        """Create a trigger input.

        Parameters
        ----------
        interface : ``expyriment.io.SerialPort`` or ``expyriment.io.ParallelPort``
            the interface to use
        default_code : int, optional
            the default code of the trigger

        """

        Input.__init__(self)
        self._interface = interface
        if default_code is not None:
            self._default_code = default_code
        else:
            self._default_code = defaults.triggerinput_default_code

    @property
    def interface(self):
        """Getter for interface"""
        return self._interface

    @property
    def default_code(self):
        """Getter for default_code"""
        return self._default_code

    @default_code.setter
    def default_code(self, value):
        """Getter for default_code"""
        self._default_code = value

    def wait(self, code=None, bitwise_comparison=False, callback_function=None,
             process_control_events=True):
        """Wait for a trigger.

        Returns the code received and the reaction time [code, rt].

        If bitwise_comparison = True, the function performs a bitwise
        comparison (logical and) between code and received input and waits
        until a certain bit pattern is set.

        Parameters
        ----------
        code : int, optional
            a specific code to wait for
        bitwise_comparison : bool, optional
            make a bitwise comparison (default = False)
        function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = True)

        Notes
        -----
        This will also by default process control events (quit and pause).
        Thus, keyboard events will be cleared from the cue and cannot be
        received by a Keyboard().check() anymore!

        See Also
        --------
        expyriment.design.Experiment.register_wait_callback_function

       """

        if _internals.skip_wait_methods:
            return None, None
        start = get_time()
        found = None
        rt = None
        if code is None:
            code = self._default_code
        self.interface.clear()
        while True:
            if isinstance(callback_function, FunctionType):
                callback_function()
            if _internals.active_exp.is_initialized:
                rtn_callback = _internals.active_exp._execute_wait_callback()
                if isinstance(rtn_callback, CallbackQuitEvent):
                    return rtn_callback, int((get_time() - start) * 1000)
                if process_control_events:
                    if _internals.active_exp.mouse.process_quit_event() or \
                       _internals.active_exp.keyboard.process_control_keys():
                        break
                else:
                    _internals.pump_pygame_events()
            read = self.interface.poll()
            if read is not None:
                if code is None: #return for every event
                    rt = int((get_time() - start) * 1000)
                    found = read
                    break
                elif compare_codes(read, code, bitwise_comparison):
                    rt = int((get_time() - start) * 1000)
                    found = read
                    break
            if Keyboard.process_control_keys():
                    break
        if self._logging:
            _internals.active_exp._event_file_log(
                            "TriggerInput,received,{0},wait".format(found))
        return found, rt

    def get_triggers(self, code=None, bitwise_comparison=False):
        """Get list of received triggers.

        For not missing any triggers the history has to be updated regularly
        (e.g. by calling this method)!
        Returns None if no history is used.

        If bitwise_comparision = True, the function performs a bitwise
        comparison (logical and) between code and received input and waits
        until a certain bit pattern is set.

        Parameters
        ----------
        code : int, optional
            a specific code to get
        bitwise_comparison : bool, optional
            make a bitwise comparison (default = False)

        """

        if self.interface.has_input_history:
            self.interface.clear()
            counter_list = []
            if code is None:
                code = self._default_code
            for event in self.interface.input_history.get_whole_buffer():
                if code is None: #get them all
                    counter_list.append(event)
                elif compare_codes(event, code, bitwise_comparison):
                    counter_list.append(event)
            return counter_list
        else:
            return None

    @property
    def trigger_count(self, code=None, bitwise_comparison=False):
        """Get the number of received triggers.

        For not missing any triggers the history has to be updated regularly
        (e.g. by calling this method)!
        Returns None if no history is used.

        If bitwise_comparision = True, the function performs a bitwise
        comparison (logical and) between code and received input and waits
        until a certain bit pattern is set.

        Parameters
        ----------
        code : int, optional
            a specific code to count
        bitwise_comparison : bool, optional
            make a bitwise comparison (default = False)

        """

        if self.interface.has_input_history:
            self.interface.clear()
            counter = 0
            if code is None:
                code = self._default_code
            for event in self.interface.input_history.get_whole_buffer():
                if code is None: #count all
                    counter += 1
                elif compare_codes(event, code, bitwise_comparison):
                    counter += 1
            return counter
        else:
            return None



"""
Default settings for the misc package.

This module contains default values for all optional arguments in the init
function of all classes in this package.

"""

__author__ = 'Florian Krause <florian@expyriment.org, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''



"""
A screen.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import pygame
try:
    import OpenGL.GL as ogl
except ImportError:
    ogl = None

from .. import _internals
from ._input_output import Output
from ..misc.geometry import position2coordinates


class Screen(Output):
    """A class implementing a screen output.

    Each experiment and all stimuli need a screen instance to function.
    They are expecting this screen instance to be referenced in
    expyriment._active_exp.screen.
    Calling expyriment.control.initialize(exp) will automatically create such
    a screen instance and will additionally reference it in exp.screen for
    easy access.

    """

    def __init__(self, colour, open_gl, window_mode, window_size, no_frame):
        """Create and set up a screen output.

        Notes
        -----
        CAUTION: We discourage from creating a screen instance manually!

        Parameters
        ----------
        colour : (int, int, int)
            colour of the screen
        open_gl : int or bool
            0/False - No OpenGL (no vsync / no blocking)
            1       - OpenGL (vsync / no blocking)
            2/True  - OpenGL (vsync / blocking)
            3       - OpenGL (vsync / alternative blocking)
        window_mode : bool
             if screen should be a window
        window_size : (int, int)
            size of the window in window_mode,
            full screen mode if size of window_mode[0]<=0
        no_frame : bool
            set True for windows (in window mode) with no frame;
            this parameter does not affect fullscreen mode

        """

        Output.__init__(self)
        self._colour = colour
        if open_gl == False:
            open_gl = 0
        elif open_gl == True:
            open_gl = 2
        self._open_gl = open_gl
        self._fullscreen = not window_mode
        self._window_size = window_size
        self._no_frame = no_frame

        if ogl is None:
            warn_message = "PyOpenGL is not installed. \
OpenGL will be deactivated!"
            print("Warning: " + warn_message)
            _internals.active_exp._event_file_warn("Screen,warning," + warn_message)
            self._open_gl = False

        pygame.display.init()
        if _internals.active_exp.is_initialized:
            self._monitor_resolution = \
                        _internals.active_exp.screen.monitor_resolution
        else:
            self._monitor_resolution = (pygame.display.Info().current_w,
                                        pygame.display.Info().current_h)
        if self._fullscreen:
            self._window_size = self._monitor_resolution
        else:
            self._window_size = window_size

        if not self._open_gl:
            if self._fullscreen:
                self._surface = pygame.display.set_mode(self._window_size,
                                                       pygame.FULLSCREEN)
            else:
                self._surface = pygame.display.set_mode(self._window_size)
                pygame.display.set_caption('Expyriment')

        else:
            try:
                pygame.display.gl_set_attribute(pygame.GL_SWAP_CONTROL, 1)
            except:
                pass

            pygame_mode = pygame.DOUBLEBUF | pygame.OPENGL
            if self._fullscreen:
                self._surface = pygame.display.set_mode(
                    self._window_size, pygame_mode | pygame.FULLSCREEN)
            else:
                if no_frame:
                    pygame_mode = pygame_mode | pygame.NOFRAME
                self._surface = pygame.display.set_mode(
                    self._window_size, pygame_mode)
                pygame.display.set_caption('Expyriment')

            ogl_version = ogl.glGetString(ogl.GL_VERSION)
            if float(ogl_version[0:3]) < 2.0:
                ogl_extensions = ogl.glGetString(ogl.GL_EXTENSIONS)
                if "ARB_texture_non_power_of_two" not in ogl_extensions:
                    raise RuntimeError("OpenGL mode is not supported on this \
machine!")
        pygame.mouse.set_visible(False)
        pygame.event.set_blocked(pygame.MOUSEMOTION)
        pygame.event.set_blocked(pygame.MOUSEBUTTONDOWN)
        pygame.event.set_blocked(pygame.MOUSEBUTTONUP)

    @property
    def colour(self):
        """Getter for colour."""
        return self._colour

    @colour.setter
    def colour(self, value):
        """Setter for colour."""
        self._colour = value

    @property
    def surface(self):
        """Getter for surface."""

        return self._surface

    @property
    def open_gl(self):
        """Getter for open_gl."""

        return self._open_gl

    @property
    def no_frame(self):
        """Getter for no_frame boolean"""

        return self._no_frame

    @property
    def window_mode(self):
        """Getter for window_mode."""

        return not self._fullscreen

    @property
    def window_size(self):
        """Getter for window_size."""

        return self._window_size

    @property
    def monitor_resolution(self):
        """Getter for monitor_resolution."""

        return self._monitor_resolution

    def update(self):
        """Update the screen.

        This will flip the display double buffer.

        """

        pygame.event.pump()
        pygame.display.flip()
        if self._open_gl >= 2:
            if self._open_gl == 3:
                ogl.glBegin(ogl.GL_POINTS)
                ogl.glColor4f(0, 0, 0, 1)  # Opaque needed for non-alpha video!
                ogl.glVertex2i(0, 0)
                ogl.glEnd()
            ogl.glFinish()
        if self._logging:
            _internals.active_exp._event_file_log("Screen,updated", 2)

    def update_stimuli(self, stimuli):
        """Update only some stimuli on the screen.

        Notes
        -----
        This does only work for non OpenGL screens.

        Parameters
        ----------
        stimuli : list
            stimulus or a list of stimuli to update

        """

        try:
            stimuli = list(stimuli)
        except:
            stimuli = [stimuli]

        if not self._open_gl:
            rectangles = []
            for stim in stimuli:
                pos = stim.absolute_position
                stim_size = stim.surface_size
                rect_pos = position2coordinates(pos, self.size)
                rect_pos[0] -= stim_size[0] // 2
                rect_pos[1] -= stim_size[1] // 2
                rectangles.append(pygame.Rect(rect_pos, stim_size))
            pygame.display.update(rectangles)
            if self._logging:
                _internals.active_exp._event_file_log("Screen,stimuli updated,{0}"\
                                .format([stim.id for stim in stimuli]), 2)
            pygame.event.pump()

    @property
    def center_x(self):
        """Getter for X-coordinate of the screen center.

        Notes
        -----
        Each initialized experiment has its one screen (exp.screen). Please use always the
        screen of your current  experiment.

        """

        return self._window_size[0] // 2

    @property
    def center_y(self):
        """Getter for the Y-coordinate of the screen center.

        Notes
        -----
        Each initialized experiment has its one screen (exp.screen). Please use always the
        screen of your current  experiment.

        """

        return self._window_size[1] // 2

    @property
    def size(self):
        """Getter for the size of the screen.

        Notes
        -----
        Each initialized experiment has its one screen (exp.screen). Please use always the
        screen of your current  experiment.

        """

        return self._window_size

    def clear(self):
        """Clear the screen.

        This will reset the default experimental background colour.

        """

        if self._open_gl:
            ogl.glClearColor(float(self._colour[0]) / 255,
                             float(self._colour[1]) / 255,
                             float(self._colour[2]) / 255, 0)
            ogl.glClear(ogl.GL_COLOR_BUFFER_BIT | ogl.GL_DEPTH_BUFFER_BIT)
        else:
            self._surface.fill(self._colour)
        if self._logging:
            _internals.active_exp._event_file_log("Screen,cleared", 2)

    def save(self, filename):
        """Save the content of the screen as a picture.

        Parameters
        ----------
        filename : str
            name of the file to write (possible extensions are BMP,
            TGA, PNG, or JPEG, with the default being TGA)

        """

        pygame.image.save(self._surface, filename)



#!/usr/bin/env python

"""
A Line stimulus.

This module contains a class implementing a line stimulus.

"""
from __future__ import absolute_import, print_function, division
from builtins import *


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import math
from copy import copy

from . import defaults
from ._visual import Visual
from ._shape import Shape
from .. import _internals
from ..misc.geometry import XYPoint, vertices_rectangle, lines_intersect, points2vertices, lines_intersection_point
from ..misc._timer import get_time


class Line(Visual):
    """A class implementing a line stimulus."""

    def __init__(self, start_point, end_point, line_width, colour=None,
                 anti_aliasing=None):
        """Create a line between two points.

        Parameters
        ----------
        start_point : (int, int)
            start point of the line (x,y)
        end_point : (int, int)
            end point of the line (x,y)
        line_width : int
            width of the plotted line
        colour : (int, int, int), optional
            line colour (int, int, int)
        anti_aliasing : int, optional
            anti aliasing parameter (good anti_aliasing with 10)

        """

        self._start_point = list(start_point)
        self._end_point = list(end_point)
        self._line_width = line_width
        Visual.__init__(self, position=[0,0]),
        if colour is None:
            colour = defaults.line_colour
            if colour is None:
                colour = _internals.active_exp.foreground_colour
        self._colour = colour
        if anti_aliasing is not None:
            self._anti_aliasing = anti_aliasing
        else:
            self._anti_aliasing = defaults.line_anti_aliasing

    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def start_point(self):
        """Getter for start_point."""
        return self._start_point

    @property
    def end_point(self):
        """Getter for end_point."""
        return self._end_point

    @property
    def line_width(self):
        """Getter for line_width."""
        return self._line_width

    @line_width.setter
    def line_width(self, value):
        """Setter for line_width."""

        if self.has_surface:
            raise AttributeError(Line._getter_exception_message.format(
                "line_width"))
        else:
            self._line_width = value

    @property
    def colour(self):
        """Getter for colour."""
        return self._colour

    @colour.setter
    def colour(self, value):
        """Setter for colour."""

        if self.has_surface:
            raise AttributeError(Line._getter_exception_message.format(
                "colour"))
        else:
            self._colour = value

    @property
    def anti_aliasing(self):
        """Getter for anti_aliasing."""

        return self._anti_aliasing

    @anti_aliasing.setter
    def anti_aliasing(self, value):
        """Setter for anti_aliasing."""

        if self.has_surface:
            raise AttributeError(Line._getter_exception_message.format(
                "anti_aliasing"))
        self._anti_aliasing = value

    @property
    def position(self):
        """Getter for position."""
        return (self.start_point[0] + (self.end_point[0] - self.start_point[0]) / 2.0,
                self.start_point[1] + (self.end_point[1] - self.start_point[1]) / 2.0)

    @position.setter
    def position(self, value):
        """Setter for position."""

        self.move(offset = (value[0] - self.position[0],
                            value[1] - self.position[1]))

    def move(self, offset):
        """Moves the stimulus in 2D space.

        Parameters
        ----------
        offset : list, optional
            translation along x and y axis

        Returns
        -------
        time : int
            the time it took to execute this method

        Notes
        -----
        When using OpenGL, this can take longer then 1ms!

        """

        start = get_time()
        self._start_point = (self._start_point[0] + offset[0],
                             self._start_point[1] + offset[1])
        self._end_point = (self._end_point[0] + offset[0],
                          self._end_point[1] + offset[1])
        if self._ogl_screen is not None and (offset[0]!=0 or offset[1]!=0):
                self._ogl_screen.refresh_position()
        return int((get_time() - start) * 1000)

    def get_shape(self):
        """returns the shape representation of the line

        Returns
        -------
        shape : stimuli.Shape
            Shape representation of the line

        """

        dist = XYPoint(self._start_point).distance(XYPoint(self._end_point))
        shape = Shape(vertex_list=vertices_rectangle(size=(dist,
                                                           self.line_width)),
                      colour=self.colour, position=self.position, line_width=0,
                      anti_aliasing= self.anti_aliasing)
        diff = (self._end_point[0] - self._start_point[0],
                self._end_point[1] - self._start_point[1])
        shape.native_rotate(degree = math.atan2(diff[1], diff[0]) * -180 / math.pi)

        return shape

    def get_connecting_shape(self, other_line, sharp_corner=False):
        """returns a modified line shape that connects without gap with
        another line, if the two lines do indeed touch each other. Otherwise,
        a runtime error is raised.

        Parameters
        ----------
        other_line : stimuli.Line
            the other line to join with
        sharp_corner : boolean (optional)
            if True, corner will be sharp and not cropped

        Returns
        -------
        shape : stimuli.Shape
            Shape representation of the modified line

        Examples
        --------
        >>> line_1 = stimuli.Line((100,100), (140, 190), line_width=20,
        >>>                       colour=misc.constants.C_BLUE)
        >>> line_2 = stimuli.Line((140, 190), (260, 230), line_width=20,
        >>>                       colour=misc.constants.C_BLUE)
        >>> line1_mod = line_1.get_connecting_shape(line_2, sharp_corner=False)

        >>> #plot
        >>> bl = stimuli.BlankScreen()
        >>> line_2.plot(bl)
        >>> line_1.plot(bl)
        >>> bl.present()
        >>> exp.keyboard.wait()

        >>> bl = stimuli.BlankScreen()
        >>> line_2.plot(bl)
        >>> line1_mod.plot(bl)
        >>> bl.present()
        >>> exp.keyboard.wait()

        """

        self_shape = self.get_shape()
        other_shape = other_line.get_shape()

        for a_end, b_start in ((True, True), (True, False), (False, False), (False, True)):
            rtn = Line.__join_line_shapes(self_shape, other_shape, a_end=a_end, b_start=b_start, sharp_corner=sharp_corner)
            if rtn is not None:
                return rtn

        raise RuntimeWarning("The two lines do not connected and can't be joined.")


    @staticmethod
    def __join_line_shapes(line_shape_a, line_shape_b, a_end=True, b_start=True, sharp_corner=False):
        """helper function: returns the modified line_shape_a"""

        a_points = line_shape_a.xy_points_on_screen
        if a_end:
            id_a_edge = (1, 2)
        else:
            id_a_edge = (3, 0)

        b_points = line_shape_b.xy_points_on_screen
        if b_start:
            id_b_edge = (3, 0)
        else:
            id_b_edge = (1, 2)

        if lines_intersect(a_points[id_a_edge[0]], a_points[id_a_edge[1]],
                           b_points[id_b_edge[0]], b_points[id_b_edge[1]]):
            #two line edges intersect
            if line_shape_a.overlapping_with_position(b_points[id_b_edge[0]].tuple):
                id_b_joinpoint = id_b_edge[1]
            else:
                id_b_joinpoint = id_b_edge[0]


            a_modified = copy(a_points)
            if a_end:
                # 0, 1, contact point, 2, 3
                id_insert = 2
            else:
                # 0, 1, 2, 3, contact point
                id_insert = 4
            a_modified.insert(id_insert, b_points[id_b_joinpoint]) # insert contact point

            if sharp_corner:
                # calc sharp corner point before contact point
                if line_shape_b.overlapping_with_position(a_points[id_a_edge[0]].tuple):
                    id_outer_edge_point = id_a_edge[1]
                else:
                    id_outer_edge_point = id_a_edge[0]
                if id_outer_edge_point == 0 or id_outer_edge_point ==1:
                    a_long_edge = (a_points[0], a_points[1])
                else:
                    a_long_edge = (a_points[2], a_points[3])

                if id_b_joinpoint == 0 or id_b_joinpoint ==1:
                    b_long_edge = (b_points[0], b_points[1])
                else:
                    b_long_edge = (b_points[2], b_points[3])
                sharp_corner_point = lines_intersection_point(a_long_edge[0], a_long_edge[1],
                                                              b_long_edge[0], b_long_edge[1])
                if sharp_corner_point is not None:
                    a_modified.insert(id_insert, sharp_corner_point)
                    id_insert +=1 # because it is later used to determin the join point
            rtn = Shape(colour=line_shape_a.colour,
                        anti_aliasing=line_shape_a.anti_aliasing,
                        vertex_list=tuple(points2vertices(a_modified)))

            rtn.move((b_points[id_b_joinpoint].x - rtn.xy_points_on_screen[id_insert].x,
                     b_points[id_b_joinpoint].y - rtn.xy_points_on_screen[id_insert].y))
            return rtn
        else:
            return None


    def _create_surface(self):
        """Create the surface of the stimulus."""

        return self.get_shape()._create_surface()


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        p1 = (-180, 15)
        p2 = (200, 0)
        line = Line(p1, p2, 2)
        line.present()
        if exp is None:
            exp_.clock.wait(1000)



from __future__ import absolute_import, print_function, division
from builtins import *

# We deliberately delay importing the inpout32 module until we try
# to use it - this allows us to import the class on machines
# which don't have it and then worry about dealing with
# using the right one later

class PParallelInpOut32(object):
    """
    This class provides read/write access to the parallel port on a PC
    using inpout32 (for instance for Windows 7 64-bit)
    """
    def __init__(self, address=0x0378):
        """Set the memory address of your parallel port,
        to be used in subsequent calls to this object

        common port addresses::

            LPT1 = 0x0378 or 0x03BC
            LPT2 = 0x0278 or 0x0378
            LPT3 = 0x0278
        """

        #from numpy import uint8
        from ctypes import windll

        if isinstance(address, str) and address.startswith('0x'): #convert u"0x0378" into 0x0378
            self.base = int(address, 16)
        else:
            self.base = address
        try:
            self.port = windll.inpout32
        except:
            self.port = windll.inpoutx64

        #BYTEMODEMASK = uint8(1 << 5 | 1 << 6 | 1 << 7)
        BYTEMODEMASK = (1 << 5 | 1 << 6 | 1 << 7) & 255

        # Put the port into Byte Mode (ECP register)
        #self.port.Out32( self.base + 0x402,
        #            int((self.port.Inp32(self.base + 0x402) & ~BYTEMODEMASK) | (1 << 5)) )
        self.port.Out32( self.base + 0x402,
                    int((self.port.Inp32(self.base + 0x402) & (~BYTEMODEMASK & 255)) | (1 << 5)) )

        # Now to make sure the port is in output mode we need to make
        # sure that bit 5 of the control register is not set
        #self.port.Out32( self.base + 2, int(self.port.Inp32(self.base + 2) & ~uint8( 1 << 5 )) )
        #self.port.Out32( self.base + 2, int(self.port.Inp32(self.base + 2) & (~(1 << 5) & 255)) )
        self.reverse = False
        self.status = None

    @property
    def reverse(self):
        """Getter for reverse."""

        return self._reverse

    @reverse.setter
    def reverse(self, value):
        """Setter for reverse."""

        if value:
            self.port.Out32(self.base + 2, int(self.port.Inp32(self.base + 2) | ((1 << 5) & 255)))
            self._reverse = True
        else:
            self.port.Out32(self.base + 2, int(self.port.Inp32(self.base + 2) & (~(1 << 5) & 255)))
            self._reverse = False

    def setData(self, data):
        """Set the data to be presented on the parallel port (one ubyte).
        Alternatively you can set the value of each pin (data pins are pins
        2-9 inclusive) using :func:`setPin`

        examples::

            p.setData(0) #sets all pins low
            p.setData(255) #sets all pins high
            p.setData(2) #sets just pin 3 high (remember that pin2=bit0)
            p.setData(3) #sets just pins 2 and 3 high

        you can also convert base 2 to int v easily in python::

            p.setData( int("00000011",2) )#pins 2 and 3 high
            p.setData( int("00000101",2) )#pins 2 and 4 high
        """
        self.port.Out32( self.base, data )

    def setPin(self, pinNumber, state):
        """Set a desired pin to be high(1) or low(0).

        Only pins 2-9 (incl) are normally used for data output,
        but status pins (1, 14, 16, 17) can be used, too::

            parallel.setPin(3, 1)#sets pin 3 high
            parallel.setPin(3, 0)#sets pin 3 low
        """
        # I can't see how to do this without reading and writing the data
        if state:
            if pinNumber in range(2, 10):
                self.port.Out32( self.base, self.port.Inp32( self.base ) | (2**(pinNumber-2)) )
            elif pinNumber==1:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base +2 ) | 1 )
            elif pinNumber==14:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base +2 ) | 2 )
            elif pinNumber==16:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base +2 ) | 4 )
            elif pinNumber==17:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base +2 ) | 8 )
        else:
            if pinNumber in range(2, 10):
                self.port.Out32( self.base, self.port.Inp32( self.base ) & (255 ^ 2**(pinNumber-2)) )
            elif pinNumber==1:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base + 2 ) & (255 ^ 1) )
            elif pinNumber==14:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base + 2 ) & (255 ^ 2) )
            elif pinNumber==16:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base + 2 ) & (255 ^ 4) )
            elif pinNumber==17:
                self.port.Out32( self.base + 2, self.port.Inp32( self.base + 2 ) & (255 ^ 8) )

    def readData(self):
        """Return the value currently set on the data pins (2-9)"""
        return (self.port.Inp32( self.base ))

    def readPin(self, pinNumber):
        """Determine whether a desired (input) pin is high(1) or low(0).

        Pins 1-17 are currently read here
        """
        if pinNumber==10:
            return (self.port.Inp32( self.base + 1 ) >> 6) & 1   # 10 = ACK
        elif pinNumber==11:
            return (self.port.Inp32( self.base + 1 ) >> 7) & 1 # 11 = BUSY
        elif pinNumber==12:
            return (self.port.Inp32( self.base + 1 ) >> 5) & 1 # 12 = PAPER-OUT
        elif pinNumber==13:
            return (self.port.Inp32( self.base + 1 ) >> 4) & 1 # 13 = SELECT
        elif pinNumber==15:
            return (self.port.Inp32( self.base + 1 ) >> 3) & 1 # 15 = ERROR
        elif pinNumber >= 2 and pinNumber <= 9:
            return (self.port.Inp32( self.base ) >> (pinNumber - 2)) & 1
        elif pinNumber==1:
            return (self.port.Inp32( self.base + 2 ) >> 0) & 1 # 0 = STROBE
        elif pinNumber==14:
            return (self.port.Inp32( self.base + 2 ) >> 1) & 1 # 0 = LineFeed
        elif pinNumber==16:
            return (self.port.Inp32( self.base + 2 ) >> 2) & 1 # 0 = RESET
        elif pinNumber==17:
            return (self.port.Inp32( self.base + 2 ) >> 3) & 1 # 0 = Select Printer
        else:
            print('Pin %i cannot be read (by the PParallelInpOut32.readPin() yet)' % (pinNumber))



"""
An event button box.

This module contains a class implementing an event button box.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from ._streamingbuttonbox import StreamingButtonBox


class EventButtonBox(StreamingButtonBox):
    """A class implementing an event button box input."""

    def __init__(self, interface):
        """Create an event button box input.

        Compared to a StreamingButtonBox, an EventButtonBox has no baseline
        (baseline=None). The methods wait() and check() are therefore
        responsive to every incoming interface event.

        Parameters
        ----------
        interface : ``expyriment.io.SerialPort`` or ``expyriment.io.ParallelPort``
            the interface to use

        """

        StreamingButtonBox.__init__(self, interface, None)

    @property
    def baseline(self):
        """Getter for baseline"""
        return self._baseline

    @baseline.setter
    def baseline(self, value):
        """Setter for baseline."""
        print("Warning: A baseline cannot be defined for an EventButtonBox!")
        self._baseline = None



# -*- coding: utf-8 -*-
#
# Expyriment documentation build configuration file, created by
# sphinx-quickstart on Fri Apr  5 15:08:26 2013.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os, time

sys.path.append(os.path.abspath(os.path.join("../../setup.py", "..")))
from setup import get_version_info_from_file, \
    get_version_info_from_release_info, \
    get_version_info_from_git
sys.path.remove(os.path.abspath(os.path.join("../../setup.py", "..")))


# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath('.'))

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.autosummary',
              'sphinx.ext.doctest',
              #'sphinx.ext.viewcode',
              'numpydoc']

autosummary_generate = True
numpydoc_show_class_members = False

# How to install numpydoc? --> sudo easy_install numpydoc

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Expyriment'
copyright = u'{0}, Florian Krause & Oliver Lindemann'.format(
    time.localtime().tm_year)

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#

# Check if we are building/installing from a built archive/distribution
filename = os.path.abspath(os.path.join(os.path.split(__file__)[0], '..',
                                        '..', 'expyriment', '__init__.py'))
version_info = get_version_info_from_file(filename)

# If not, we are building/installing from source
if version_info[0] == '':

    # Are we building/installing from a source archive/distribution?
    version_info = get_version_info_from_release_info()
    if version_info[2].startswith("$Format:"):

        # Are we building/installing from the GitHub repository?
        try:
            from subprocess import Popen, PIPE
            proc = Popen(['git', 'rev-list', '--max-parents=0', 'HEAD'],
                         stdout=PIPE, stderr=PIPE)
            initial_revision = proc.stdout.readline()
            if not b'e21fa0b4c78d832f40cf1be1d725bebb2d1d8f10' in \
                                                            initial_revision:
                raise Exception
            version_info = get_version_info_from_git()
        except:
          raise RuntimeError("Building documentation failed!")

# The release name
release = version_info[0]

# The normal X.Y.Z version.
version_nr = release.split("-")[0]  #release[:5]

# The short X.Y version.
version = ".".join(version_nr.split(".")[:2])  #version_nr[:3]

rst_epilog = """
.. |expyriment-wheel| replace:: expyriment-{0}-py2.py3-none-any.whl
.. |expyriment-wheel-code| replace:: ``expyriment-{0}-py2.py3-none-any.whl``
""".format(release)


# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme' # 'default', 'sphinxdoc',  'haiku' 

# Temporary work-around for spacing problem between parameter and parameter
# type in the doc, see https://github.com/numpy/numpydoc/issues/215. The bug
# has been fixed in sphinx (https://github.com/sphinx-doc/sphinx/pull/5976) but
# through a change in sphinx basic.css except rtd_theme does not use basic.css.
# In an ideal world, this would get fixed in this PR:
# https://github.com/readthedocs/sphinx_rtd_theme/pull/747/files
def setup(app):
    app.add_stylesheet("basic.css")

html4_writer = True

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {
#    "sidebarbgcolor": "#222222",
#    "relbarbgcolor": "#222222",
#    "linkcolor": "#ff9632",
#    "visitedlinkcolor": "#ff9632",
#    "sidebarlinkcolor": "#ff9632",
#    "relbarlinkcolor": "#a046fa",
#    "sidebartextcolor": "#969696",
#    "relbartextcolor": "#a046fa",
#    "footerbgcolor": "#222222",
#    "footertextcolor": "#969696",
#    "headbgcolor": "#dddddd",
#    "headtextcolor": "black",
#
#}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = "xpy_button.png"

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = 'favicon.ico'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []  #['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {'**': ['globaltoc.html', 'searchbox.html']}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_domain_indices = ['py-modindex']

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'Expyrimentdoc'


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'Expyriment.tex', u'Expyriment Documentation',
   u'Florian Krause \\& Oliver Lindemann', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Additional stuff for the LaTeX preamble.
#latex_preamble = ''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'expyriment', u'Expyriment Documentation',
     [u'Florian Krause & Oliver Lindemann'], 1)
]



"""
Secure hashes from files
"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import sys
from os.path import sep
from re import split
from hashlib import sha1
from copy import copy
from ._miscellaneous import is_interactive_mode

def _make_secure_hash(filename):
    """returns secure hash from file or None, if not possile"""

    try:
        with open(filename, 'br') as f:
            return sha1(f.read()).hexdigest()[:6]
    except:
        return None

def get_experiment_secure_hash():
    """Returns the fingerprint, that is, the first six places of the secure
    hash (sha1) of the main file of the current experiment.

    Returns
    -------
    hash: string or None
        first six places of the experiment secure hash or None, if no main
        file can be found

    Notes
    -----
    Fingerprints of experiments help to ensure that the correct version is
    running in the lab. Hash codes are written to all output files and
    printed in the command line output. If you want to check post hoc the
    version of your experiment, create the secure hash (sha1) of your
    expyriment .py-file and compare the first six place with the code in the
    output file.

    """

    if main_file in secure_hashes:
        return secure_hashes[main_file]
    else:
        return None

def get_module_hash_dictionary():
    """Returns a dictionary with the fingerprints of all modules imported
    from the local folded.

    Returns
    -------
    hashes: dict
        hash dict with all imported modules
        keys = file names, values = sha hashes

    Notes
    -----
    See get_experiment_secure_hash() for further information about Expyriment
    secure hashes.

    """
    if main_file in secure_hashes:
        rtn = copy(secure_hashes)
        rtn.pop(main_file)
        return rtn
    else:
        return {}

def _append_hashes_from_imported_modules(hash_dict, filename):
    """append hashes from imported modules und returns hash_dict"""
    try:
        with open(filename) as f:
            for line in f:
                line = line.strip()
                if line.startswith("from"):
                    modules = [line.split(" ")[1]]
                elif line.startswith("import"):
                    modules = split(" |,", line)[1:]
                else:
                    modules = []

                modules = [x.replace(".", sep) for x in modules]

                for module in modules:
                    pyfile = module + ".py"
                    if pyfile not in hash_dict:
                        sha = _make_secure_hash(pyfile)
                        if sha is not None:
                            hash_dict[pyfile] = sha
                            hash_dict = _append_hashes_from_imported_modules(
                                        hash_dict, pyfile) #recursion
    except:
        pass
    return hash_dict

def module_hashes_as_string():
    """helper function that converts dict to str"""
    if len(secure_hashes)>1:
        txt = ""
        for fl, sha in get_module_hash_dictionary().items():
            txt += u"{0} ({1}), ".format(fl, sha)
        return txt[:-2]
    else:
        return ""

def cout_hashes():
    """helper function that prints hash information"""
    if get_experiment_secure_hash() is not None:
        print("Main file: {0} ({1})".format(main_file,
                            get_experiment_secure_hash()))
        if len(secure_hashes)>1:
            print("Modules: " + module_hashes_as_string())

# print hash information when imported
if not is_interactive_mode():
    main_file = sys.argv[0]
else:
    main_file = ""
secure_hashes = {main_file : _make_secure_hash(main_file)}
secure_hashes = _append_hashes_from_imported_modules(secure_hashes, main_file)
cout_hashes()



"""An emulation of the csv.reader module.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


def reader(the_file):
    '''
    This is a 'dirty' emulation of the csv.reader module only used for 
    Expyriment loading designs under Android. The function reads in a csv file
    and returns a 2 dimensional array.
    
    Parameters
    ----------
    the_file: iterable
        The file to be parsed.
    
    Notes
    -----
    It is strongly suggested the use, if possible, the csv package from the 
    Python standard library.
   
    
    '''
    delimiter = ","
    rtn = []
    for row in the_file:
        rtn.append([strn.strip() for strn in row.split(delimiter)])
    return rtn




"""The expyriment clock.

This module contains an experimental clock.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import sys
import time
from types import FunctionType

from ._timer import get_time
from .. import _internals


class Clock(object) :
    """Basic timing class.

    Unit of time is milliseconds.

    """


    def __init__(self, sync_clock=None):
        """Create a clock.

        Parameters
        ----------
        sync_clock : misc.Clock, optional
            synchronise clock with existing one

        """

        if (sync_clock.__class__.__name__ == "Clock"):
            self.__init_time = sync_clock.init_time // 1000
        else:
            self.__init_time = get_time()

        self._init_localtime = time.localtime()
        self.__start = get_time()


    @staticmethod
    def monotonic_time():
        """Returns the time of the high-resolution monotonic timer that is
        used by Expyriment internally.

        """
        return get_time()

    @property
    def init_time(self):
        """Getter for init time in milliseconds."""

        return self.__init_time * 1000

    @property
    def time(self):
        """Getter for current time in milliseconds since clock init."""

        return int((get_time() - self.__init_time) * 1000)

    @property
    def cpu_time(self):
        """OBSOLETE property: Please use monotonic_time()."""
        # TODO: make deprecated with 1.0

        return get_time()

    @property
    def stopwatch_time(self):
        """Getter for time in milliseconds since last reset_stopwatch.

        The use of the stopwatch does not affect the clock time.
        """

        return int((get_time() - self.__start) * 1000)

    @property
    def init_localtime(self):
        """Getter for init time in local time"""

        return self._init_localtime

    def reset_stopwatch(self):
        """"Reset the stopwatch.

        The use of the stopwatch does not affect the clock time.
        """

        self.__start = get_time()

    def wait(self, waiting_time, callback_function=None, process_control_events=False):
        """Wait for a certain amount of milliseconds.

        Parameters
        ----------
        waiting_time : int
            time to wait in milliseconds
        function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = False)

        Returns
        -------
        quit_event : expyriment.control.CallbackQuitEvent object
           the callback quit even in case a wait function has been registered

        See Also
        --------
        expyriment.design.Experiment.register_wait_callback_function

        """

        if _internals.skip_wait_methods:
            return
        start = self.time
        if isinstance(callback_function, FunctionType) or \
           (process_control_events or \
             _internals.active_exp.is_callback_registered):
            while (self.time < start + waiting_time):
                if isinstance(callback_function, FunctionType):
                    rtn_callback = callback_function()
                    if isinstance(rtn_callback, _internals.CallbackQuitEvent):
                        return rtn_callback
                if _internals.active_exp.is_initialized:
                    rtn_callback = _internals.active_exp._execute_wait_callback()
                    if isinstance(rtn_callback, _internals.CallbackQuitEvent):
                        return rtn_callback
                    if process_control_events:
                        if _internals.active_exp.keyboard.process_control_keys():
                            break
                        _internals.active_exp.mouse.process_quit_event()
                    else:
                        _internals.pump_pygame_events()
        else:
            looptime = 200
            if (waiting_time > looptime):
                if _internals.active_exp.is_initialized:
                    while (self.time < start + (waiting_time - looptime)):
                        if process_control_events:
                            if _internals.active_exp.mouse.process_quit_event() or \
                               _internals.active_exp.keyboard.process_control_keys():
                                break
                        else:
                            import pygame
                            pygame.event.pump()
                else:
                    time.sleep((waiting_time - looptime) // 1000)
            while (self.time < start + waiting_time):
                pass

    def wait_seconds(self, time_sec, callback_function=None,
                     process_control_events=False):
        """Wait for a certain amount of seconds.

        Parameters
        ----------
        time_sec : int
            time to wait in seconds
        callback_function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = False)

        Returns
        -------
        quit_event : expyriment.control.CallbackQuitEvent object
           the callback quit even in case a wait function has been registered

        See Also
        --------
        wait
        expyriment.design.Experiment.register_wait_callback_function

        """

        return self.wait(time_sec * 1000, callback_function, process_control_events)

    def wait_minutes(self, time_minutes, callback_function=None,
                     process_control_events=False):
        """Wait for a certain amount of minutes.

        Parameters
        ----------
        time_minutes : int
            time to wait in minutes
        callback_function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = False)

        Returns
        -------
        quit_event : expyriment.control.CallbackQuitEvent object
           the callback quit even in case a wait function has been registered

        See Also
        --------
        wait
        expyriment.design.Experiment.register_wait_callback_function

        """

        return self.wait_seconds(time_minutes * 60, callback_function,
                                 process_control_events)



"""Python library for cognitive and neuroscientific experiments.

Expyriment is an open-source and platform independent light-weight Python
library for designing and conducting timing-critical behavioural and
neuroimaging experiments. The major goal is to provide a well-structured
Python library for a script-based experiment development with a high priority
on the readability of the resulting programme code. Due to the availability of
an Android runtime environment, Expyriment is also suitable for the
development of experiments running on tablet PCs or smart-phones.

Expyriment has been tested extensively under Linux and Windows and is an
all-in-one solution, as it handles stimulus presentation, the recording of
input/output events, communication with other devices, and the collection
and preprocessing of data. Furthermore, it offers a hierarchical design
structure, which allows for an intuitive transition from the experimental
design to a running program. It is therefore also suited for students, as well
as for experimental psychologists and neuroscientists with little programming
experience.

Website: http://www.expyriment.org

To cite Expyriment in publications, please refer to the following article:

  Krause, F., & Lindemann, O. (2014). Expyriment: A Python library for cognitive
  and neuroscientific experiments. *Behavior Research Methods*, 46(2), 416-428.

  see http://dx.doi.org/10.3758/s13428-013-0390-6

"""

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


try:
    import future as _future
    if int(_future.__version__.split(".")[1]) < 15:
      raise RuntimeError("Expyriment {0} ".format(__version__) +
                      "is not compatible with Future {0}".format(
                        _future.__version__) +
                      "\nPlease install Future >= 0.15.")
except ImportError:
    raise ImportError("Expyriment {0} ".format(__version__) +
                      "needs the package 'Future')." +
                      "\nPlease install Future >= 0.15.")

try:
    import os
    os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"
    import pygame as _pygame
    if _pygame.vernum < (1, 9, 1):
        raise RuntimeError("Expyriment {0} ".format(__version__) +
                      "is not compatible with Pygame {0}.{1}.{2}.".format(
                                                    _pygame.vernum) +
                      "\nPlease install Pygame 1.9.")
except ImportError:
    raise ImportError("Expyriment {0} ".format(__version__) +
                      "needs the package 'Pygame')." +
                      "\nPlease install Pygame 1.9.")

try:
    import OpenGL as _OpenGL
    if int(_OpenGL.version.__version__[0]) < 3:
        raise RuntimeError("Expyriment {0} ".format(__version__) +
                      "is not compatible with PyOpenGL {0}.{1}.{2}.".format(
                        int(_OpenGL.version.__version__[0]),
                        int(_OpenGL.version.__version__[2]),
                        int(_OpenGL.version.__version__[4]),
                          ) +
                      "\nPlease install PyOpenGL 3.0.")
except ImportError:
    print("No OpenGL support!" +
                    "\nExpyriment {0} ".format(__version__) +
                      "needs the package 'PyOpenGL'."
                      "\nPlease install PyOpenGL 3.0 for OpenGL functionality.")

import sys as _sys
from ._internals import get_version, import_all_extras
from ._internals import PYTHON3 as _PYTHON3

if not( (_sys.version_info[0] == 2 and _sys.version_info[1] >= 6) or
        (_PYTHON3 and _sys.version_info[1] >= 3) ):
    raise RuntimeError("Expyriment {0} ".format(__version__) +
                      "is not compatible with Python {0}.{1}.".format(
                                                    _sys.version_info[0],
                                                    _sys.version_info[1]) +
                      "\nPlease use Python 2.6+ or Python 3.3+.")
else:
    print("Expyriment {0} ".format(get_version()))

# Check if local 'test.py{c|o|d}' shadows 'test' package of standard library
try:
    import imp as _imp
    import os as _os
    import sys as _sys
    for _package in ["test"]:
        _tf = _os.path.abspath(_imp.find_module(_package)[1])
        _mf = _os.path.abspath(_os.path.abspath(_sys.argv[0]))
        if _os.path.split(_tf)[0] == _os.path.split(_mf)[0] or \
                _os.path.split(_tf)[0] == _os.path.abspath(_os.path.curdir):
                    _m = "Warning: "
                    _m += "'{0}' is shadowing package '{1}'!"
                    print(_m.format(_tf, _package))
except:
    pass

from . import _internals
from . import design
from . import misc
from . import stimuli
from . import io
from . import control

if not misc.is_android_running():
    from ._api_reference_tool import show_documentation

# add extras folder if it exists
if _internals.get_plugins_folder() is not None:
    _sys.path.append(_internals.get_plugins_folder())

# post import hook
exec(_internals.post_import_hook())




"""
The control._miscellaneous module of expyriment.

"""
from __future__ import absolute_import, print_function, division
from builtins import *


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import sys
from types import FunctionType

import pygame

from . import defaults
from .. import _internals
from .._internals import CallbackQuitEvent
from ..misc import is_idle_running, is_ipython_running


def start_audiosystem():
    """Start the audio system.

    Notes
    ------
    The audiosystem is automatically started when initializing an
    Experiment!

    """

    pygame.mixer.init()


def stop_audiosystem():
    """Stop the audio system."""

    pygame.mixer.quit()


def get_audiosystem_is_playing(channel=None):
    """Check if the audiosystem is busy playing sounds.

    Parameters
    ----------
    channel : pygame.mixer.Channel, optional
        specific channel to check

    Returns
    -------
    out : bool
        Returns True if any sound is playing.

    """

    if channel is None:
        rtn = pygame.mixer.get_busy()
    else:
        rtn = channel.get_busy()
    if rtn == 0:
        rtn = False
    elif rtn > 0:
        rtn = True
    return rtn


def wait_end_audiosystem(channel=None, callback_function=None,
                         process_control_events=True):
    """Wait until audiosystem has ended playing sounds.

    Blocks until the audiosystem is not busy anymore and only returns then.

    Parameters
    ----------
    channel : pygame.mixer.Channel, optional
        specific channel to wait for end of playing
    callback_function : function, optional
        function to repeatedly execute during waiting loop
    process_control_events : bool, optional
        process ``io.Keyboard.process_control_keys()`` and
        ``io.Mouse.process_quit_event()`` (default = True)

    Notes
    ------
    This will also by default process control events (quit and pause).
    Thus, keyboard events will be cleared from the cue and cannot be
    received by a Keyboard().check() anymore!

    """

    from .. import io
    while get_audiosystem_is_playing(channel):
        if _internals.skip_wait_methods:
            break
        if isinstance(callback_function, FunctionType):
            rtn_callback = callback_function()
            if isinstance(rtn_callback, CallbackQuitEvent):
                if channel is None:
                    pygame.mixer.stop()
                else:
                    channel.stop()
                return rtn_callback
        if _internals.active_exp.is_initialized:
            rtn_callback = _internals.active_exp._execute_wait_callback()
            if isinstance(rtn_callback, CallbackQuitEvent):
                if channel is None:
                    pygame.mixer.stop()
                else:
                    channel.stop()
                return rtn_callback
            if process_control_events:
                if _internals.active_exp.mouse.process_quit_event() or \
                   _internals.active_exp.keyboard.process_control_keys():
                    if channel is None:
                        pygame.mixer.stop()
                    else:
                        channel.stop()
                    break
            else:
                pygame.event.pump()


def set_skip_wait_methods(on=True):
    """Skip all wait methods.

    Parameters
    ----------
    on : bool, optional
        If True, all wait methods in the experiment (i.e. all wait functions
        in ``expyriment.io`` and the clock) will be omitted (default = True)

    """

    if on:
        _internals.skip_wait_methods = True
    else:
        _internals.skip_wait_methods = False


def set_develop_mode(on=True, intensive_logging=False, skip_wait_methods=False):
    """Set defaults for a more convenient develop mode.

    Notes
    -----
    The function set the following global variables

    >>> expyriment.control.defaults.initialize_delay = 0
    >>> expyriment.control.defaults.window_mode = True
    >>> expyriment.control.defaults.fast_quit = True
    >>> expyriment.control.defaults.auto_create_subject_id = True
    >>> expyriment.io.defaults.outputfile_time_stamp = False

    Parameters
    ----------
    on : bool, optional
        set develop_mode on (True) or off (False)
    intensive_logging : bool, optional
        True sets expyriment.io.defaults.event_logging=2
        (default = False)
    skip_wait_methods : bool, optional
        If True, all wait functions in the experiment (i.e. all wait functions
        in ``expyriment.io`` and the clock) will be omitted (default = False)

    """

    from .. import io
    if on:
        defaults._mode_settings = [defaults.initialize_delay,
                                   defaults.window_mode,
                                   defaults.fast_quit,
                                   io.defaults.outputfile_time_stamp,
                                   defaults.auto_create_subject_id]
        if intensive_logging:
                defaults._mode_settings.append(defaults.event_logging)
                defaults.event_logging = 2
        if skip_wait_methods:
            set_skip_wait_methods(True)
        print("*** DEVELOP MODE ***")
        defaults.initialize_delay = 0
        defaults.window_mode = True
        defaults.fast_quit = True
        io.defaults.outputfile_time_stamp = False
        defaults.auto_create_subject_id = True
    else:
        print("*** NORMAL MODE ***")
        if defaults._mode_settings is not None:
            defaults.initialize_delay = defaults._mode_settings[0]
            defaults.window_mode = defaults._mode_settings[1]
            defaults.fast_quit = defaults._mode_settings[2]
            io.defaults.outputfile_time_stamp = \
                defaults._mode_settings[3]
            defaults.auto_create_subject_id = defaults._mode_settings[4]
            defaults._mode_settings = None
            try:
                defaults.event_logging = defaults._mode_settings[5]
            except:
                pass
            set_skip_wait_methods(False)
        else:
            pass  # Nothing to do


def _get_module_values(goal_dict, module):
    value = None
    namespace = locals()
    for var in dir(module):
        if not var.startswith("_"):
            exec("value = {0}.{1}".format(module.__name__, var), namespace)
            goal_dict["{0}.{1}".format(module.__name__, var)] = namespace['value']
    return goal_dict


def get_defaults(search_str="", as_string=False):
    """Return a dictionary with all default values in the current Expyriment
    environment. The keys represent the variables names.

    Parameters
    ----------
    search_str : str, optional
        search for a specific expression
    as_string : bool, optional
        print as string instead of dict

    """

    from .. import design,stimuli, io, control,misc
    from ..io import extras as ioextras
    from ..design import extras as designextras
    from ..stimuli import extras as stimuliextras
    from ..misc import extras as miscextras

    defaults = {}
    defaults = _get_module_values(defaults, design.defaults)
    defaults = _get_module_values(defaults, control.defaults)
    defaults = _get_module_values(defaults, stimuli.defaults)
    defaults = _get_module_values(defaults, io.defaults)
    defaults = _get_module_values(defaults, misc.defaults)
    defaults = _get_module_values(defaults, designextras.defaults)
    defaults = _get_module_values(defaults, stimuliextras.defaults)
    defaults = _get_module_values(defaults, ioextras.defaults)
    defaults = _get_module_values(defaults, miscextras.defaults)
    if len(search_str) >= 0:
        tmp = {}
        for key in list(defaults.keys()):
            if key.lower().find(search_str.lower()) >= 0:
                tmp[key] = defaults[key]
        defaults = tmp
    if as_string:
        sorted_keys = list(defaults.keys())
        sorted_keys.sort()
        rtn = ""
        for key in sorted_keys:
            tabs = "\t" * (4 - int((len(key) + 1) // 8))
            rtn += key + ":" + tabs + repr(defaults[key]) + "\n"
    else:
        rtn = defaults

    return rtn


def register_wait_callback_function(function, exp=None):
    """Register a wait callback function.

    The registered wait callback function will be repetitively executed in
    all Expyriment wait and event loops that wait for an external input.
    That is, they are executed by the following functions (at least once!):

        - control.wait_end_audiosystem
        - misc.Clock.wait
        - misc.Clock.wait_seconds
        - misc.Clock.wait_minutes
        - io.Keyboard.wait
        - io.Keyboard.wait_char
        - io.Mouse.wait_press
        - io.SerialPort.read_line
        - io.StreamingButtonBox.wait
        - io.EventButtonBox.wait
        - io.GamePad.wait_press
        - io.TriggerInput.wait
        - io.TextInput.get
        - io.TouchScreenButtonBox.wait
        - io.extras.CedrusResponseDevice.wait
        - stimuli.Video.wait_frame
        - stimuli.Video.wait_end

    Parameters
    ----------
    function : function
        the wait function
    exp : design.Experiment, optional
        specific experiment for which to register wait function

    Notes
    -----
    CAUTION! If wait callback function takes longer than 1 ms to process,
    Expyriment timing will be affected!

    See Also
    --------
    unregister_wait_callback_function

    """

    if exp is not None:
        exp.register_wait_callback_function(function)
    else:
        _internals.active_exp.register_wait_callback_function(function)


def unregister_wait_callback_function(exp=None):
    """Unregister all wait callback functions.

    Parameters
    ----------
    exp : design.Experiment, optional
        specific experiment for which to unregister the wait callback function


    See Also
    --------
    register_wait_callback_function

    """

    if exp is not None:
        exp.unregister_wait_callback_function()
    else:
        _internals.active_exp.unregister_wait_callback_function()

def _set_stdout_logging(event_file):
    """Set logging of stdout and stderr to event file.

    Note that if the script is called via IPython or IDLE logging will not
    work.

    Parameters
    ----------
    event_file : io.EventFile
        the event file

    """

    class Logger(object):
        def __init__(self, event_file, log_tag):
            self.terminal = sys.stdout
            self.event_file = event_file
            self.tag = log_tag
            self._buffer = []

        def write(self, message):
            self.terminal.write(message)
            self._buffer.append(message)
            if message.endswith("\n"):
                tmp = "".join(self._buffer).strip("\n")
                self.event_file.log("{0},received,{1}".format(self.tag,
                                                              repr(tmp)))
                self._buffer = []

        def flush(self):  # required for some modules (e.g. multiprocessing)
            pass

    if is_ipython_running():
        print("Standard output and error logging is switched off under IPython.")
    elif is_idle_running():
        print("Standard output and error logging is switched off under IDLE.")
    else:
        sys.stderr = Logger(event_file, "stderr")
        sys.stdout = Logger(event_file, "stdout")



#!/usr/bin/env python

"""
Obsolete stimuli

Calls are merely defined to give user appropriate error feedback

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

class Dot(object):
    """OBSOLETE CLASS: Please use Circle!"""

    def __init__(self, radius=None, colour=None, position=None):
        """OBSOLETE CLASS: Please use Circle!

        """

        raise DeprecationWarning("Dot is an obsolete class. Please use Circle!")


class Frame(object):
    """OBSOLETE CLASS: Please use Rectangle!"""

    def __init__(self, size=None, position=None, frame_line_width=None,
                 colour=None, anti_aliasing=None, line_width=None):
        """OBSOLETE CLASS: Please use Rectangle with a line_width > 0!

        """
        raise DeprecationWarning("Frame is an obsolete class. Please use Rectangle with a line_width > 0!")


"""
An event buffer.

This module contains a class implementing an event buffer.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from ._clock import Clock
from .. import _internals


class Buffer(object):
    """A class implementing a general event buffer."""

    def __init__(self, name="Buffer", clock=None):
        """Create an event buffer.

        Parameters
        ----------
        name : str
            the name of the buffer
        clock : expyriment.Clock object (optional)
            an experimental clock

        """

        self._name = name
        if clock is not None:
            self._clock = clock
        else:
            if _internals.active_exp.is_initialized:
                self._clock = _internals.active_exp.clock
            else:
                self._clock = Clock()
        self._memory = []

    @property
    def name(self):
        """Getter for name"""
        return self._name

    @property
    def clock(self):
        """Getter for clock"""
        return self._clock

    @property
    def memory(self):
        """Getter for memory [list of tuples (code, rt)]"""
        return self._memory

    def add_event(self, event):
        """Add an event to the buffer.

        Parameters
        ----------
        event : anything
            the event to add

        """

        item = (event, self._clock.time)
        self._memory.append(item)

    def add_events(self, events):
        """Add a list of events to the buffer.

        All events get the same time stamp!

        Parameters
        ----------
        events : list
            the event list to add

        """
        ts = [self._clock.time] * len(events)
        self._memory.extend(list(zip(events, ts)))

    def get_size(self):
        """Return the number of elements in the buffer.
        
        Returns
        -------
        size : int
            the size of the buffer
    
        """

        return len(self._memory)

    def get_element(self, position):
        """Get an element (code, rt) from the buffer. 

        Parameters
        ----------
        position : int
            the position to get the element from
            
        Returns
        -------
        element : (anything, float)
            The element to get from the buffer
            
        """

        if position < len(self._memory) and position >= 0:
            element = self._memory[position]
        else:
            element = (0, 0)
        return element

    def clear(self):
        """Clear the buffer."""

        self._memory = []

    def get_last_event(self):
        """Get the last event (code, rt) in the buffer.
        
        Returns
        -------
        element : (anything, float)
            the last element of the buffer
            
        """

        if len(self._memory) > 0:
            element = self._memory[-1]
        else:
            element = [0, 0]
        return element

    def get_whole_buffer(self):
        """Get a copy of the buffer.
        
        Returns
        -------
        copy : list
            the copy of the buffer
            
        """

        bcopy = self._memory[:]
        return bcopy


class ByteBuffer(Buffer):
    """A class implementing a buffer for bytes.

    The ByteBuffer class is also used for the input_histoy of serial and
    parallel ports.

    """

    def __init__(self, name="ByteBuffer", clock=None):
        """Create a buffer for bytes.

        Parameters
        ----------
        name : str
            the name of the buffer
        clock : expyriment.Clock object (optional)
            an experimental clock

        """

        Buffer.__init__(self, name, clock)

    def check_set(self, search_byte, search_start_position=0):
        """
        Check if bits are set in buffer bytes and return position.

        Parameters
        ----------
        search_byte : int
            the byte to search for
        search_start_position : int (optional)
            position to start search from (int)
        
        Returns
        -------
        position : int
            the position of element with set bits
            
        """

        found_pos = 0
        pos = search_start_position
        for x in range(pos, len(self._memory)):
            elem = self._memory[x]
            # OR: at least one of the bits is set
            if (elem[0] & search_byte) > 0:
                found_pos = x
        return found_pos

    def check_unset(self, search_byte, search_start_position=0):
        """
        Check if bits are NOT set in buffer bytes and return position.

        Parameters
        ----------
        search_byte : int
            the byte to search for
        search_start_position : int (optional)
            position to start search from
            
        Returns
        -------
        position : int
            the position of element with unset bits

        """

        found_pos = 0
        pos = search_start_position
        for x in range(pos, len(self._memory)):
            elem = self._memory[x]
            # OR: at least one of the bits is set
            if (elem[0] ^ 255) & search_byte:
                found_pos = x
        return found_pos

    def check_value(self, value, search_start_position=0):
        """
        Check if value is in buffer bytes and return the position.

        Parameters
        ----------
        value : int
            the value to check
        search_start_position : int (optional)
            position to start search from
            
        Returns
        -------
        position : int
            the position of element with value

        """

        found_pos = 0
        pos = search_start_position
        for x in range(pos, len(self._memory)):
            elem = self._memory[x]
            if elem[0] == value:
                found_pos = x
        return found_pos



# We deliberately delay importing the pyparallel module until we try
# to use it - this allows us to import the class on machines
# which don't have it and then worry about dealing with
# using the right one later

# This is necessary to stop the local parallel.py masking the module
# we actually want to find!
from __future__ import absolute_import, print_function, division
from builtins import *

# We duck-type the parallel port objects
class PParallelLinux(object):
    """
    This class provides read/write access to the parallel port for linux using
    pyparallel.

    Note that you must have the lp module removed and the ppdev module loaded
    to use this code::
        sudo rmmod lp
        sudo modprobe ppdev
    """
    def __init__(self, address='/dev/parport0'):
        """Set the device node of your parallel port

        common port addresses::

            LPT1 = /dev/parport0
            LPT2 = /dev/parport1
            LPT3 = /dev/parport2"""
        import parallel as pyp

        if not hasattr(pyp, 'Parallel'):
            # We failed to import pyparallel properly
            # We probably ended up with psychopy.parallel instead...
            raise Exception('Failed to import pyparallel - is it installed?')

        self.port = pyp.Parallel(address)
        self.reverse = False
        self.status = None

    def __del__(self):
        if hasattr(self, 'port'):
            del self.port

    @property
    def reverse(self):
        """Getter for reverse."""

        return self._reverse

    @reverse.setter
    def reverse(self, value):
        """Setter for reverse."""

        if value:
            self.port.PPDATADIR(False)
            self._reverse = True
        else:
            self.port.PPDATADIR(True)
            self._reverse = False

    def setData(self, data):
        """Set the data to be presented on the parallel port (one ubyte).
        Alternatively you can set the value of each pin (data pins are pins
        2-9 inclusive) using :func:`~psychopy.parallel.setPin`

        examples::

            p.setData(0) #sets all pins low
            p.setData(255) #sets all pins high
            p.setData(2) #sets just pin 3 high (remember that pin2=bit0)
            p.setData(3) #sets just pins 2 and 3 high

        you can also convert base 2 to int v easily in python::

            parallel.setData( int("00000011",2) )#pins 2 and 3 high
            parallel.setData( int("00000101",2) )#pins 2 and 4 high
        """
        self.port.setData(data)

    def setPin(self, pinNumber, state):
        """Set a desired pin to be high(1) or low(0).

        Only pins 2-9 (incl) are normally used for data output::

            p.setPin(3, 1)#sets pin 3 high
            p.setPin(3, 0)#sets pin 3 low
        """
        # I can't see how to do this without reading and writing the data
        if state:
            if pinNumber in range(2, 10):
                self.port.setData(self.port.PPRDATA() | (2**(pinNumber-2)))
            elif pinNumber==1:
                self.port.PPWCONTROL(self.port.PPRCONTROL() | 1)
            elif pinNumber==14:
                self.port.PPWCONTROL(self.port.PPRCONTROL() | 2)
            elif pinNumber==16:
                self.port.PPWCONTROL(self.port.PPRCONTROL() | 4)
            elif pinNumber==17:
                self.port.PPWCONTROL(self.port.PPRCONTROL() | 8)
        else:
            if pinNumber in range(2, 10):
                self.port.setData(self.port.PPRDATA() & (255 ^ 2**(pinNumber-2)))
            elif pinNumber==1:
                self.port.PPWCONTROL(self.port.PPRCONTROL() & (255 ^ 1))
            elif pinNumber==14:
                self.port.PPWCONTROL(self.port.PPRCONTROL() & (255 ^ 2))
            elif pinNumber==16:
                self.port.PPWCONTROL(self.port.PPRCONTROL() & (255 ^ 4))
            elif pinNumber==17:
                self.port.PPWCONTROL(self.port.PPRCONTROL() & (255 ^ 8))

    def readData(self):
        """Return the value currently set on the data pins (2-9)"""
        return (self.port.PPRDATA())

    def readPin(self, pinNumber):
        """Determine whether a desired (input) pin is high(1) or low(0).

        Pins 1-17 are currently read here """
        if pinNumber==10:
            return self.port.getInAcknowledge()
        elif pinNumber==11:
            return self.port.getInBusy()
        elif pinNumber==12:
            return self.port.getInPaperOut()
        elif pinNumber==13:
            return self.port.getInSelected()
        elif pinNumber==15:
            return self.port.getInError()
        elif pinNumber>=2 and pinNumber <=9:
            return (self.port.PPRDATA() >> (pinNumber - 2)) & 1
        elif pinNumber==1:
            return (self.port.PPRCONTROL() >> 0) & 1 # 0 = STROBE
        elif pinNumber==14:
            return (self.port.PPRCONTROL() >> 1) & 1 # 0 = LineFeed
        elif pinNumber==16:
            return (self.port.PPRCONTROL() >> 2) & 1 # 0 = RESET
        elif pinNumber==17:
            return (self.port.PPRCONTROL() >> 3) & 1 # 0 = Select Printer
        else:
            print('Pin %i cannot be read (by PParallelLinux.readPin() yet)' % (pinNumber))



#!/usr/bin/env python

"""
A text line stimulus.

This module contains a class implementing a text line stimulus.

"""
from __future__ import absolute_import, division, print_function
from builtins import (ascii, bytes, chr, dict, filter, hex, input,
                      int, map, next, oct, pow, range, round,
                      str, super, zip) # without open, because
                      # pygame.font.Font needs old file object under PY2


__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import os

import pygame

from . import defaults
from ._visual import Visual
from ..misc import find_font, unicode2byte, byte2unicode
from .. import _internals


# Keep track of open file handles and previously loaded font objects
open_filehandles = []
fonts = {}


class TextLine(Visual):
    """A class implementing a single text line."""

    def __init__(self, text, position=None, text_font=None, text_size=None,
                 text_bold=None, text_italic=None, text_underline=None,
                 text_colour=None, background_colour=None, max_width=None):
        """Create a text line stimulus.

        NOTE: text_font can be both, a name or path to a font file!
        When text_font is a name, Expyriment will try to find a font that
        best matches the given name.
        If no matching font can be found, or if the given font file cannot be
        found, the Pygame system default will be used.
        In any case the value of the attribute text_font will always
        resemble the font that is actually in use!

        Parameters
        ----------
        text : str
            text to show (str)
        position : (int, int), optional
            position of the stimulus
        text_font : str, optional
            font to use as name or as path to a font file
        text_size : int, optional
            text size
        text_bold : bool, optional
            font should be bold
        text_italic : bool, optional
            font should be italic
        text_underline : bool, optional
            font should get an underline
        text_colour : (int, int, int), optional
            text colour
        background_colour : (int, int, int), optional
            background colour
        max_width: int, optional
            maximum surface width of the text line stimulus
            if this parameter is defined, text lines that exceed this
            surface width will be trimmed (indicated by a '~' as last letter)

        """

        pygame.font.init()
        if position is None:
            position = defaults.textline_position
        Visual.__init__(self, position, log_comment=text)
        self._text = text
        self._max_width = max_width
        if text_size is None:
            text_size = defaults.textline_text_size
        if text_size is not None:
            self._text_size = text_size
        else:
            self._text_size = _internals.active_exp.text_size
        if text_font is None:
            text_font = defaults.textline_text_font
        if text_font is None:
            text_font = _internals.active_exp.text_font
        if text_font is None:
            text_font = "FreeSans"
        self._text_font = find_font(text_font)
        try:
            with open(self._text_font, 'rb') as f:
                pygame.font.Font(f, 10)
        except:
            raise IOError("Font '{0}' not found!".format(text_font))
        if text_bold is not None:
            self._text_bold = text_bold
        else:
            self._text_bold = defaults.textline_text_bold
        if text_italic is not None:
            self._text_italic = text_italic
        else:
            self._text_italic = defaults.textline_text_italic
        if text_underline is not None:
            self._text_underline = text_underline
        else:
            self._text_underline = defaults.textline_text_underline
        if text_colour is None:
            text_colour = defaults.textline_text_colour
        if text_colour is not None:
            self._text_colour = text_colour
        else:
            self._text_colour = _internals.active_exp.foreground_colour
        if background_colour is not None:
            self._background_colour = background_colour
        else:
            self._background_colour = \
                defaults.textline_background_colour

    _getter_exception_message = "Cannot set {0} if surface exists!"

    @property
    def text(self):
        """Getter for text."""

        return self._text

    @text.setter
    def text(self, value):
        """Setter for text."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text"))
        else:
            self._text = value

    @property
    def text_font(self):
        """Getter for text_font."""

        return self._text_font

    @text_font.setter
    def text_font(self, value):
        """Setter for text_font."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text_font"))
        else:
            self._text_font = value

    @property
    def text_size(self):
        """Getter for text_size."""

        return self._text_size

    @text_size.setter
    def text_size(self, value):
        """Setter for text_size."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text_size"))
        else:
            self._text_size = value

    @property
    def text_bold(self):
        """Getter for text_bold."""

        return self._text_bold

    @text_bold.setter
    def text_bold(self, value):
        """Setter for text_bold."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text_bold"))
        else:
            self._text_bold = value

    @property
    def text_italic(self):
        """Getter for text_italic."""

        return self._text_italic

    @text_italic.setter
    def text_italic(self, value):
        """Setter for text_italic."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text_italic"))
        else:
            self._text_italic = value

    @property
    def text_underline(self):
        """Getter for text_underline."""

        return self._text_underline

    @text_underline.setter
    def text_underline(self, value):
        """Setter for text_underline."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text_underline"))
        else:
            self._text_underline = value

    @property
    def text_colour(self):
        """Getter for text_colour."""

        return self._text_colour

    @text_colour.setter
    def text_colour(self, value):
        """Setter for text_colour."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "text_colour"))
        else:
            self._text_colour = value

    @property
    def background_colour(self):
        """Getter for background_colour."""

        return self._background_colour

    @background_colour.setter
    def background_colour(self, value):
        """Setter for background_colour."""

        if self.has_surface:
            raise AttributeError(TextLine._getter_exception_message.format(
                "background_colour"))
        else:
            self._background_colour = value

    def _create_surface(self):
        """Create the surface of the stimulus."""

        # Due to a bug in handling file names in PyGame 1.9.2, we pass a file
        # handle to PyGame. See also:
        # https://github.com/expyriment/expyriment/issues/81
        if os.path.isfile(self._text_font):
            # We preserve previously created fonts, so that we don't open the
            # same font over and over again.
            if (self._text_font, self._text_size) in fonts:
                _font = fonts[(self._text_font, self._text_size)]
            else:
                f = open(self._text_font, 'rb')
                open_filehandles.append(f)
                _font = pygame.font.Font(f, self._text_size)
                fonts[(self._text_font, self._text_size)] = _font
        else:
            _font = pygame.font.Font(self._text_font, self._text_size)

        _font.set_bold(self.text_bold)
        _font.set_italic(self.text_italic)
        _font.set_underline(self.text_underline)
        if not _internals.is_unicode_string(self.text):
            # Pygame wants latin-1 encoding here for character strings
            _text = byte2unicode(self.text).encode('latin-1')
        else:
            _text = self.text
        if self.background_colour:
            surface = _font.render(_text, True, self.text_colour,
                                   self.background_colour)
        else:
            surface = _font.render(_text, True, self.text_colour)
        surface = surface.convert_alpha()

        if self._max_width is not None and self._max_width > 0 and surface.get_size()[0] > self._max_width:
            # trim too long text lines
            self._text = self._text[:-2] + "~"
            surface = self._create_surface() # recursion

        return surface


    @staticmethod
    def _demo(exp=None):
        if exp is None:
            from .. import control
            control.set_develop_mode(True)
            control.defaults.event_logging = 0
            exp_ = control.initialize()
        textline = TextLine("abcde fghijk lmnopqrstuvwxyz 12 348 56789",
                            text_font="Helvetica",
                            text_size=20, text_bold=False)
        textline.present()
        if exp is None:
            exp_.clock.wait(1000)



"""
Input and output serial port.

This module contains a class implementing serial port input/output.

"""

from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org> \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


import atexit
from types import ModuleType, FunctionType

try:
    import serial
    from serial.tools.list_ports import comports as list_com_ports
except:
    serial = None
    list_com_ports = None

from . import defaults
from ._input_output import Input, Output
from .. import _internals, misc
from .._internals import CallbackQuitEvent, PYTHON3


class SerialPort(Input, Output):
    """A class implementing a serial port input and output."""

    def __init__(self, port, baudrate=None, bytesize=None, parity=None,
                 stopbits=None, timeout=None, xonxoff=None, rtscts=None,
                 dsrdtr=None, input_history=None, os_buffer_size=None,
                 clock=None):
        """Create a serial port input and output.

        The port argument will accept the number of the port (e.g. 0 for COM1)
        as well as a string describing the full port location ("COM1" or
        "/dev/ttyS0").

        Notes
        -----
        An input_history can be used to overcome the size limitation of the
        receive buffer of the operating system. An input_history consists of a
        misc.ByteBuffer instance.
        In order to not miss any input, the serial port has to be updated
        regularly (i.e. calling read_input() or clear() before the receive
        buffer will be full). If the receive buffer size is set correctly, a
        warning will be given, when the input_history was not updated fast
        enough.
        Importantly, the fuller the receive buffer is, the longer clearing
        and polling will take (this can be more than 1 ms!), since all the
        bytes have to be transferred to the input_history.

        Parameters
        ----------
        port : int or str
            the port name
        baudrate : int, optional
        bytesize : int, optional
        parity : str, optional
            'E'=even, 'O'=odd, 'N'=none
        stopbits : int, optional
        timeout : int, optional
            the timeout for read(): -1=block
        xonxoff : int, optional
        rtscts : int, optional
        dsrdtr : int, optional
        input_history : bool, optional
            True if an input_history should be used
        os_buffer_size : int, optional
            the size of the receive input_history provided by the operating
            system in bytes
        clock : misc.Clock, optional
            an experimental clock
                            (optional)

        """

        if not isinstance(serial, ModuleType):
            message = """SerialPort can not be initialized.
The Python package 'pySerial' is not installed."""
            raise ImportError(message)

        serial_version = tuple(map(lambda x: int(x), serial.VERSION.split(".")))
        if serial_version < (2, 5):
            raise ImportError("Expyriment {0} ".format(__version__) +
                    "is not compatible with PySerial {0}.".format(
                        serial.VERSION) +
                      "\nPlease install PySerial 2.5 or higher.")

        Input.__init__(self)
        Output.__init__(self)
        if baudrate is None:
            baudrate = defaults.serialport_baudrate
        if bytesize is None:
            bytesize = defaults.serialport_bytesize
        if parity is None:
            parity = defaults.serialport_parity
        if stopbits is None:
            stopbits = defaults.serialport_stopbits
        if timeout is None:
            timeout = defaults.serialport_timeout
        if timeout == -1:
            timeout = None
        if xonxoff is None:
            xonxoff = defaults.serialport_xonxoff
        if rtscts is None:
            rtscts = defaults.serialport_rtscts
        if dsrdtr is None:
            dsrdtr = defaults.serialport_dsrdtr
        if clock is not None:
            self._clock = clock
        else:
            if _internals.active_exp.is_initialized:
                self._clock = _internals.active_exp.clock
            else:
                self._clock = misc.Clock()
        if input_history is None:
            input_history = defaults.serialport_input_history
        if input_history is True:
            self._input_history = misc.ByteBuffer(
                name="SerialPortBuffer (Port {0})".format(repr(port)),
                clock=self._clock)
        else:
            self._input_history = False
        if os_buffer_size is None:
            os_buffer_size = defaults.serialport_os_buffer_size
        self._os_buffer_size = os_buffer_size

        self._serial = serial.Serial(port, baudrate, bytesize, parity,
                                    stopbits, timeout, xonxoff, rtscts,
                                    dsrdtr)
        if not self._serial.isOpen():
            raise IOError("Could not open serial port")

        atexit.register(self.close)

    @property
    def input_history(self):
        """Getter for input_history."""

        return self._input_history

    @property
    def clock(self):
        """Getter for clock."""
        return self._clock

    @property
    def serial(self):
        """Getter for serial."""
        return self._serial

    @property
    def os_buffer_size(self):
        """Getter for os_buffer_size."""
        return self._os_buffer_size

    @property
    def baudrate(self):
        """Getter for baudrate."""

        return self._serial.baudrate

    @baudrate.setter
    def baudrate(self, value):
        """Setter for baudrate."""

        self._serial.baudrate = value

    @property
    def bytesize(self):
        """Getter for bytesize."""

        return self._serial.bytesize

    @bytesize.setter
    def bytesize(self, value):
        """Setter for bytesize."""

        self._serial.bytesize = value

    @property
    def parity(self):
        """Getter for parity."""

        return self._serial.parity

    @parity.setter
    def parity(self, value):
        """Setter for parity."""

        self._serial.parity = value

    @property
    def stopbits(self):
        """Getter for stopbits."""

        return self._serial.stopbits

    @stopbits.setter
    def stopbits(self, value):
        """Setter for stopbits."""

        self._serial.stopbits = value

    @property
    def timeout(self):
        """Getter for timeout."""

        return self._serial.timeout

    @timeout.setter
    def timeout(self, value):
        """Setter for timeout."""

        self._serial.timeout = value

    @property
    def xonxoff(self):
        """Getter for xonxoff."""

        return self._serial.xonxoff

    @xonxoff.setter
    def xonxoff(self, value):
        """Setter for xonxoff."""

        self._serial.xonxoff = value


    @property
    def rtscts(self):
        """Getter for rtscts."""

        return self._serial.rtscts

    @rtscts.setter
    def rtscts(self, value):
        """Setter for rtscts."""

        self._serial.rtscts = value

    @property
    def dsrdtr(self):
        """Getter for dsrdtr."""

        return self._serial.dsrdtr

    @dsrdtr.setter
    def dsrdtr(self, value):
        """Setter for dsrdtr."""

        self._serial.dsrdtr = value

    def close(self):
        """Close the serial port."""

        try:
            self._serial.close()
        except:
            pass

    @property
    def has_input_history(self):
        """Returns if a input_history exists or not (True / False)."""

        return isinstance(self._input_history, misc.ByteBuffer)

    def clear(self, skip_input_history=False):
        """Clear the serial port.

        Notes
        -----
        If an input_history is used, all data in the receive buffer, will be
        added to the history before clearing (via read_input()).

        The copy process might take a few milliseconds. If you need a
        very fast clearing of the device buffer, you should skip copying the
        data into the input_history using the skip_input_history parameter.

        Parameters
        ----------
        skip_input_history : bool, optional
            if True available data will not be copied to the input_history.
            (default = False)

        """

        if self.has_input_history and not skip_input_history:
            self.read_input()
        else:
            self._serial.flushInput()
        if self._logging:
            _internals.active_exp._event_file_log("SerialPort {0},cleared".\
                           format(repr(self._serial.port)), 2)

    def read_input(self):
        """Read all input from serial port.

        If a input_history is used, all received data will be added.

        Returns
        -------
        out : list of int
            a list of integer values of the received bytes

        """

        read_time = self._clock.time
        read = bytes(self._serial.read(self._serial.inWaiting()))
        if len(read) > 0:
            read = list(read)
            if self.has_input_history:
                if len(self._input_history.memory):
                    last_time = self._input_history.memory[-1][1]
                else:
                    last_time = 0 #first time
                self._input_history.add_events(read)
                if len(read) >= (self._os_buffer_size - 1) and last_time:
                    warn_message = "{0} not updated for {1} ms!".format(
                                        self._input_history.name,
                                        read_time - last_time)
                    print("Warning: " + warn_message)
                    _internals.active_exp._event_file_warn(warn_message)
            if self._logging:
                _internals.active_exp._event_file_log(
                        "SerialPort {0}, read input, {1} bytes".format(
                        repr(self._serial.port), len(read)), 2)
            return read
        return []

    def poll(self):
        """Poll the serial port.

        If a input_history is used, it will be added.

        Returns
        -------
        out : int
             a single integer value of the received byte
        """

        poll_time = self._clock.time
        read = bytes(self._serial.read())
        if read != b"":
            if self.has_input_history:
                last = self._input_history.get_last_event()
                self._input_history.add_event(list(read)[0])
                if last[1] > 0: # if input_history is empty
                    if self._serial.inWaiting() >= (self._os_buffer_size - 1):
                        warn_message = "{0} not updated for {1} ms!".format(
                                            self._input_history.name,
                                            poll_time - last[1])
                        print("Warning: " + warn_message)
                        _internals.active_exp._event_file_warn(warn_message)
            if self._logging:
                _internals.active_exp._event_file_log(
                        "SerialPort {0},received,{1},poll".format(
                            repr(self._serial.port), list(read)[0]), 2)
            return ord(read)
        return None

    def read_line(self, duration=None, callback_function=None,
                  process_control_events=True):
        """Read a line from serial port (until newline) and return byte string.

        The function is waiting for input. Use the duration parameter
        to avoid too long program blocking.

        Parameters
        ----------
        duration : int, optional
            try to read for given amount of time (default=None)
        callback_function : function, optional
            function to repeatedly execute during waiting loop
        process_control_events : bool, optional
            process ``io.Keyboard.process_control_keys()`` and
            ``io.Mouse.process_quit_event()`` (default = True)

        Returns
        -------
        line : bytes or str

        Notes
        -----
        This will also by default process control events (quit and pause).
        Thus, keyboard events will be cleared from the cue and cannot be
        received by a Keyboard().check() anymore!

        See Also
        --------
        expyriment.design.Experiment.register_wait_callback_function

        """

        if _internals.skip_wait_methods:
            return

        rtn_string = b""
        if duration is not None:
            timeout_time = self._clock.time + duration

        if self._logging:
            _internals.active_exp._event_file_log(
                    "SerialPort {0}, read line, start".format(
                    repr(self._serial.port)), 2)

        while True:
            if isinstance(callback_function, FunctionType):
                rtn_callback = callback_function()
                if isinstance(rtn_callback, CallbackQuitEvent):
                    rtn_string = rtn_callback
                    break
            if _internals.active_exp.is_initialized:
                rtn_callback = _internals.active_exp._execute_wait_callback()
                if isinstance(rtn_callback, CallbackQuitEvent):
                    rtn_string = rtn_callback
                    break
                if process_control_events:
                    if _internals.active_exp.mouse.process_quit_event() or \
                       _internals.active_exp.keyboard.process_control_keys():
                        break
                else:
                    _internals.pump_pygame_events()

            byte = self.poll()
            if byte:
                if byte != 10 and byte != 13:
                    rtn_string = rtn_string + bytes([byte])
                elif byte == 10:
                    break
                elif byte == 13:
                    pass
            elif duration is not None and self._clock.time >= timeout_time:
                break
        if self._logging:
            _internals.active_exp._event_file_log(
                "SerialPort {0}, read line, end".format(
                    repr(self._serial.port)), 2)

        return rtn_string

    @staticmethod
    def get_available_ports():
        """Return a list of strings representing the available serial ports.

        Notes
        -----
        If pyserial is not installed, 'None' will be returned.

        Returns
        -------
        arr : list
            a list of strings representing the available serial ports

        """

        if not isinstance(serial, ModuleType):
            return None
        ports = sorted([x[0] for x in list_com_ports()])
        return ports

    def send(self, data):
        """Send data via the serial port.

        Parameters
        ----------
        data : int
            the data to be sent

        """

        self._serial.write(bytes([data]))

        if self._logging:
            _internals.active_exp._event_file_log("SerialPort {0},sent,{1}"\
                                .format(repr(self._serial.port), data), 2)

    def send_line(self, data, carriage_return=False, line_feed=True):
        """Send a line of data via the serial port.

        Parameters
        ----------
        data : bytes or str
            the data to be sent
        carriage_return : bool, optional
            whether to add a carriage return at end of data (default=True)
        line_feed : bool, optional
            whether to add a line feed at end of data (default=True)

        """

        if PYTHON3:
            data = list(data)
        else:
            data = [ord(x) for x in data]

        if carriage_return:
            data.append(13)
        if line_feed:
            data.append(10)

        if self._logging:
            _internals.active_exp._event_file_log(
                    "SerialPort {0}, send line, start".format(
                    repr(self._serial.port)), 2)

        for x in data:
            self.send(x)

        if self._logging:
            _internals.active_exp._event_file_log(
                "SerialPort {0}, send line, end".format(
                    repr(self._serial.port)), 2)

    @staticmethod
    def _self_test(exp):
        """Test the serial port."""

        from .. import io, stimuli
        def int2bin(n, count=8):
            return "".join([str((n >> y) & 1) for y in range(count - 1, -1, -1)])

        result = {}
        result["testsuite_serial_port"] = ""
        result["testsuite_serial_baudrate"] = ""
        result["testsuite_serial_parity"] = ""
        result["testsuite_serial_stopbits"] = ""
        result["testsuite_serial_success"] = "No"

        ports = io.SerialPort.get_available_ports()
        if ports is None:
            while True:
                stimuli.TextScreen(
                                "The Python package 'pySerial' is not installed!",
                                   "[Press RETURN to continue]").present()
                key, rt_ = exp.keyboard.wait(misc.constants.K_RETURN)
                if key is not None:
                    break
            return result

        elif ports == []:
            while True:
                stimuli.TextScreen("No serial ports found!",
                                   "[Press RETURN to continue]").present()
                key, rt_ = exp.keyboard.wait(misc.constants.K_RETURN)
                if key is not None:
                    break
            return result

        else:
            import serial

            idx = io.TextMenu("Select Serial Port", ports, width=200,
                                         justification=1, scroll_menu=5).get()
            comport = ports[idx]

            rates = [0, 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400,
                     4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800,
                     500000, 576000, 921600, 1000000, 1152000, 1500000, 2000000,
                     2500000, 3000000, 3500000, 4000000]
            idx = io.TextMenu("Select Baudrate", [repr(x) for x in rates],
                                         width=200, justification=1,
                                         scroll_menu=2).get(preselected_item=14)
            baudrate = rates[idx]

            parities = ["N", "E", "O"]
            idx = io.TextMenu("Select Parity", parities, width=200,
                                         justification=1, scroll_menu=2).get(
                                             preselected_item=0)
            parity = parities[idx]

            stopbits = [0, 1, 1.5, 2]
            idx = io.TextMenu("Select Stopbits", [repr(x) for x in stopbits],
                              width=200, justification=1,
                              scroll_menu=2).get(preselected_item=1)
            stopbit = stopbits[idx]

            try:
                ser = io.SerialPort(port=comport, baudrate=baudrate,
                                    parity=parity, stopbits=stopbit,
                                    input_history=True)
            except serial.SerialException:
                while True:
                    stimuli.TextScreen("Could not open {0}!".format(comport),
                                       "[Press RETURN to continue]").present()
                    key, rt_ = exp.keyboard.wait(misc.constants.K_RETURN)
                    if key is not None:
                        break
                result["testsuite_serial_port"] = comport
                result["testsuite_serial_baudrate"] = baudrate
                result["testsuite_serial_parity"] = parity
                result["testsuite_serial_stopbits"] = stopbit
                result["testsuite_serial_success"] = "No"
                return result

            s = stimuli.TextScreen("Serial Port {0}".format(comport), "")
            s.present()
            cnt = 0
            while True:
                read = ser.read_input()
                if len(read) > 0:
                    byte = read[-1]
                    cnt = ser.input_history.get_size()
                    if byte is not None:
                        stimuli.TextScreen("Serial Port {0}".format(comport),
                                      "{0}\n {1} - {2}".format(cnt, byte,
                                       int2bin(byte))).present()
                key = exp.keyboard.check(misc.constants.K_RETURN)
                if key is not None:
                    break
                else:
                    s.present()

            result["testsuite_serial_port"] = comport
            result["testsuite_serial_baudrate"] = baudrate
            result["testsuite_serial_parity"] = parity
            result["testsuite_serial_stopbits"] = stopbit
            result["testsuite_serial_success"] = "Yes"
            return result



"""The misc package.

This package contains miscellaneous classes, modules and functions.

See also expyriment.misc.extras for more misc.

"""

__author__ = 'Florian Krause <florian@expyriment.org> \
              Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from . import defaults
from ._miscellaneous import compare_codes, add_fonts, list_fonts, find_font
from ._miscellaneous import unicode2byte, byte2unicode, numpad_digit_code2ascii
from ._miscellaneous import get_monitor_resolution, is_idle_running, is_ipython_running, \
                            is_android_running, is_interactive_mode, create_colours, \
                            has_internet_connection, which, download_from_stash,\
                            py2py3_sort_array, round
from ._get_system_info import get_system_info
from ._secure_hash import get_module_hash_dictionary, get_experiment_secure_hash, module_hashes_as_string
from . import constants
from ._buffer import  Buffer, ByteBuffer
from ._clock import Clock
from . import geometry
from . import data_preprocessing
from . import statistics
from ._colour import Colour

from os import path as _path
add_fonts(byte2unicode(_path.abspath(_path.join(_path.dirname(__file__),
                  "..", "_fonts"))))



"""
The control package of expyriment.

"""

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from . import defaults
from ._miscellaneous import start_audiosystem, stop_audiosystem, \
        get_audiosystem_is_playing, wait_end_audiosystem, \
        set_develop_mode, get_defaults, register_wait_callback_function, \
        unregister_wait_callback_function
from .._internals import CallbackQuitEvent
from ._experiment_control import initialize, start, pause, end
from ._test_suite import run_test_suite



"""
The permute module.

This module implements permutation of blocks, trials and conditions.

"""

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''


from ._permute import is_permutation_type
from ._permute import balanced_latin_square, cycled_latin_square



"""
Get System Information.
"""
from __future__ import absolute_import, print_function, division
from future import standard_library
standard_library.install_aliases()
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import sys
import os
import platform
import subprocess
import socket
try:
    import OpenGL as _ogl
except ImportError:
    _ogl = None
try:
    import serial as _serial
    from serial.tools.list_ports import comports as _list_com_ports
    _com_port_list = _list_com_ports()
except ImportError:
    _serial = None
    _com_port_list = []
try:
    import parallel as _parallel
except ImportError:
    _parallel = None
except WindowsError:
    _parallel = None
try:
    import numpy as _numpy
except:
    _numpy = None
try:
    import mediadecoder as _mediadecoder
except:
    _mediadecoder = None
try:
    import sounddevice as _sounddevice
except:
    _sounddevice = None
try:
    import PIL.Image as _pil
except:
    _pil = None
import pygame

from .._internals import PYTHON3

def _get_registry_value(key, subkey, value):
    if PYTHON3:
        import winreg as _winreg  # TODO check me on Windows
    else:
        import _winreg
    key = getattr(_winreg, key)
    handle = _winreg.OpenKey(key, subkey)
    (value, type) = _winreg.QueryValueEx(handle, value)
    return value


def get_system_info(as_string=False):
    """Print system information to standard out and return as a dictionary.

    Parameters
    ----------
    as_string : boolean, optional
        Print as string instead of dict (default = False)

    """

    from ..io import SerialPort, ParallelPort
    from .._internals import get_settings_folder

    info = {}

    # Get platform specific info for Linux
    if sys.platform.startswith("linux"):
        os_platform = "Linux"
        os_name = platform.linux_distribution()[0]
        details = []
        if "XDG_CURRENT_DESKTOP" in os.environ:
            details.append(os.environ["XDG_CURRENT_DESKTOP"])
        if "DESKTOP_SESSION" in os.environ:
            details.append(os.environ["DESKTOP_SESSION"])
        if details != []:
            os_details = ", ".join(details)
        else:
            os_details = ""
        os_version = platform.linux_distribution()[1]

        try:
            hardware_cpu_details = ""
            with open('/proc/cpuinfo') as f:
                for line in f:
                    if line.startswith("model name"):
                        hardware_cpu_details = line.split(":")[1].strip()
        except:
            hardware_cpu_details = ""

        try:
            with open('/proc/meminfo') as f:
                for line in f:
                    if line.startswith("MemTotal"):
                        mem_total = \
                            int(line.split(":")[1].strip()[:-2].strip()) // 1024
                    if line.startswith("MemFree"):
                        mem_free = \
                            int(line.split(":")[1].strip()[:-2].strip()) // 1024
                    if line.startswith("Buffers"):
                        mem_buffers = \
                            int(line.split(":")[1].strip()[:-2].strip()) // 1024
                    if line.startswith("Cached"):
                        mem_cached = \
                            int(line.split(":")[1].strip()[:-2].strip()) // 1024
            hardware_memory_total = str(mem_total) + " MB"
            hardware_memory_free = str(mem_free + mem_buffers + mem_cached) + \
                                   " MB"

        except:
            hardware_memory_total = ""
            hardware_memory_free = ""

        try:
            hardware_audio_card = ""
            cards = []
            p = subprocess.Popen(['lspci'], stdout=subprocess.PIPE)
            for line in p.stdout:
                if "Audio" in line:
                    cards.append(line.split(":")[-1].strip())
            p.wait()
            hardware_audio_card = ", ".join(cards)
        except:
            try:
                hardware_audio_card = ""
                cards = []
                with open('/proc/asound/cards') as f:
                    for line in f:
                        if line.startswith(" 0"):
                            cards.append(line.split(":")[1].strip())
                hardware_audio_card = ", ".join(cards)
            except:
                hardware_audio_card = ""

        try:
            current_folder = os.path.split(os.path.realpath(sys.argv[0]))[0]
            s = os.statvfs(current_folder)
            disk_total = int((s.f_frsize * s.f_blocks) // 1024 ** 2)
            disk_free = int((s.f_frsize * s.f_bavail) // 1024 ** 2)
            hardware_disk_space_total = str(disk_total) + " MB"
            hardware_disk_space_free = str(disk_free) + " MB"
        except:
            hardware_disk_space_total = ""
            hardware_disk_space_free = ""

        try:
            hardware_video_card = ""
            cards = []
            p = subprocess.Popen(['lspci'], stdout=subprocess.PIPE)
            for line in p.stdout:
                if "VGA" in line:
                    cards.append(line.split(":")[-1].strip())
            p.wait()
            hardware_video_card = ", ".join(cards)
        except:
            hardware_video_card = ""

    # Get platform specific info for Windows
    elif sys.platform.startswith("win"):
        os_platform = "Windows"
        os_name = "Windows"
        os_details = platform.win32_ver()[2]
        os_version = platform.win32_ver()[1]

        try:
            hardware_cpu_details = str(_get_registry_value(
                "HKEY_LOCAL_MACHINE",
                "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",
                "ProcessorNameString"))
        except:
            hardware_cpu_details = ""

        try:
            import ctypes
            class MEMORYSTATUSEX(ctypes.Structure):
                _fields_ = [("dwLength", ctypes.c_uint),
                            ("dwMemoryLoad", ctypes.c_uint),
                            ("ullTotalPhys", ctypes.c_ulonglong),
                            ("ullAvailPhys", ctypes.c_ulonglong),
                            ("ullTotalPageFile", ctypes.c_ulonglong),
                            ("ullAvailPageFile", ctypes.c_ulonglong),
                            ("ullTotalVirtual", ctypes.c_ulonglong),
                            ("ullAvailVirtual", ctypes.c_ulonglong),
                            ("sullAvailExtendedVirtual", ctypes.c_ulonglong), ]

                def __init__(self):
                    # Initialize this to the size of MEMORYSTATUSEX
                    self.dwLength = 2 * 4 + 7 * 8  # size = 2 ints, 7 longs
                    return super(MEMORYSTATUSEX, self).__init__()

            stat = MEMORYSTATUSEX()
            ctypes.windll.kernel32.GlobalMemoryStatusEx(ctypes.byref(stat))
            hardware_memory_total = str(stat.ullTotalPhys // 1024 ** 2) + " MB"
            hardware_memory_free = str(stat.ullAvailPhys // 1024 ** 2) + " MB"
        except:
            hardware_memory_total = ""
            hardware_memory_free = ""

        try:
            current_folder = os.path.split(os.path.realpath(sys.argv[0]))[0]
            _, disk_total, disk_free = ctypes.c_int64(), ctypes.c_int64(), \
                                       ctypes.c_int64()
            ret = ctypes.windll.kernel32.GetDiskFreeSpaceExW(
                str(current_folder), ctypes.byref(_),
                ctypes.byref(disk_total), ctypes.byref(disk_free))
            hardware_disk_space_total = str(disk_total.value // 1024 ** 2) + " MB"
            hardware_disk_space_free = str(disk_free.value // 1024 ** 2) + " MB"
        except:
            hardware_disk_space_total = ""
            hardware_disk_space_free = ""

        try:
            tmp = str(_get_registry_value("HKEY_LOCAL_MACHINE",
                                          "HARDWARE\\DEVICEMAP\\VIDEO",
                                          "\Device\Video0"))
            idx = tmp.find("System")
            card = str(_get_registry_value("HKEY_LOCAL_MACHINE",
                                           tmp[idx:].replace("\\", "\\\\"),
                                           "Device Description"))
            hardware_video_card = card
        except:
            hardware_video_card = ""

        hardware_audio_card = ""  # TODO

    # Get platform specific info for OS X
    elif sys.platform.startswith("darwin"):
        os_platform = "Darwin"
        os_name = "Mac OS X"
        os_details = platform.mac_ver()[1][1]
        os_version = platform.mac_ver()[0]
        try:
            proc = subprocess.Popen(['sysctl', '-a', 'hw.model'],
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE)
            hardware_cpu_details = \
                    proc.stdout.readline().split(":")[1].strip()
        except:
            hardware_cpu_details = ""
        try:
            proc = subprocess.Popen(['sysctl', '-a', 'hw.memsize'],
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE)
            mem_total = int(proc.stdout.readline().split(":")[1].strip()) // 1024 ** 2
            hardware_memory_total = str(mem_total) + " MB"
        except:
            hardware_memory_total = ""

        try:
            import re
            proc = subprocess.Popen(['vm_stat'],
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE)
            output = proc.stdout.read().split("\n")
            for item in output:
                x = item.find("Pages active:")
                y = item.find("page size of")
                if x > -1:
                    non_decimal = re.compile(r'[^\d.]+')
                    active = int(non_decimal.sub('', item).strip("."))
                if y > -1:
                    non_decimal = re.compile(r'[^\d.]+')
                    page = int(non_decimal.sub('', item).strip("."))
            hardware_memory_free = str(mem_total - (active * page) // 1024 ** 2) + " MB"

        except:
            hardware_memory_free = ""

        try:
            current_folder = os.path.split(os.path.realpath(sys.argv[0]))[0]
            s = os.statvfs(current_folder)
            disk_total = int((s.f_frsize * s.f_blocks) // 1024 ** 2)
            disk_free = int((s.f_frsize * s.f_bavail) // 1024 ** 2)
            hardware_disk_space_total = str(disk_total) + " MB"
            hardware_disk_space_free = str(disk_free) + " MB"
        except:
            hardware_disk_space_total = ""
            hardware_disk_space_free = ""

        try:
            proc = subprocess.Popen(['system_profiler', 'SPAudioDataType'],
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE)
            hardware_audio_card = \
                proc.stdout.read().split("\n")[2].strip(":").strip()
        except:
            hardware_audio_card = ""

        try:
            import plistlib
            proc = subprocess.Popen(['system_profiler', 'SPDisplaysDataType',
                                     '-xml'],
                                    stdout=subprocess.PIPE,
                                    stdin=subprocess.PIPE)
            pl = plistlib.readPlist(proc.stdout)
            hardware_video_card = []
            for card in pl[0]['_items']:
                hardware_video_card.append(card['sppci_model'])
        except:
            hardware_video_card = ""

    # Fill in info
    info["os_architecture"] = platform.architecture()[0]
    info["os_name"] = os_name
    info["os_details"] = os_details
    info["os_platform"] = os_platform
    info["os_release"] = platform.release()
    info["os_version"] = os_version
    info["settings_folder"] = get_settings_folder()
    info["python_expyriment_build_date"] = __date__
    info["python_expyriment_revision"] = __revision__
    info["python_expyriment_version"] = __version__
    if _numpy is not None:
        numpy_version = _numpy.version.version
    else:
        numpy_version = ""
    info["python_numpy_version"] = numpy_version
    if _mediadecoder is not None:
        mediadecoder_version = _mediadecoder.__version__
    else:
        mediadecoder_version = ""
    info["python_mediadecoder_version"] = mediadecoder_version
    if _sounddevice is not None:
        sounddevice_version = _sounddevice.__version__
    else:
        sounddevice_version = ""
    info["python_sounddevice_version"] = sounddevice_version
    if _pil is not None:
        pil_version = _pil.__version__
    else:
        pil_version = ""
    info["python_pil_version"] = pil_version
    info["python_pygame_version"] = pygame.version.ver
    if _ogl is not None:
        pyopengl_version = _ogl.version.__version__
    else:
        pyopengl_version = ""
    info["python_pyopengl_version"] = pyopengl_version
    if _parallel is not None:
        parallel_version = _parallel.VERSION
    else:
        parallel_version = ""
    info["python_pyparallel_version"] = parallel_version
    if _serial is not None:
        serial_version = _serial.VERSION
    else:
        serial_version = ""
    info["python_pyserial_version"] = serial_version
    info["python_version"] = "{0}.{1}.{2}".format(sys.version_info[0],
                                                  sys.version_info[1],
                                                  sys.version_info[2])

    info["hardware_audio_card"] = hardware_audio_card
    info["hardware_cpu_architecture"] = platform.machine()
    info["hardware_cpu_details"] = hardware_cpu_details
    info["hardware_cpu_type"] = platform.processor()
    info["hardware_disk_space_free"] = hardware_disk_space_free
    info["hardware_disk_space_total"] = hardware_disk_space_total
    try:
        socket.gethostbyname("google.com")
        hardware_internet_connection = "Yes"
    except:
        hardware_internet_connection = "No"
    info["hardware_internet_connection"] = hardware_internet_connection
    info["hardware_memory_total"] = hardware_memory_total
    info["hardware_memory_free"] = hardware_memory_free
    info["hardware_ports_parallel"] = \
            ParallelPort.get_available_ports()
    info["hardware_ports_parallel_driver"] = ParallelPort.get_driver()
    try:
        info["hardware_ports_serial"] = [x.device for x in _com_port_list]
    except:
        info["hardware_ports_serial"] = _com_port_list
    info["hardware_video_card"] = hardware_video_card

    if as_string:
        sorted_keys = list(info.keys())
        sorted_keys.sort()
        rtn = ""
        for key in sorted_keys:
            tabs = "\t" * (4 - int((len(key) + 1) // 8))
            try:
                rtn += key + ":" + tabs + info[key] + "\n"
            except TypeError:
                rtn += key + ":" + tabs + repr(info[key]) + "\n"
    else:
        rtn = info

    return rtn



#!/usr/bin/env python

"""A command line interface for Expyriment.

The Expyriment command line interface provides a collection of convenient
methods helpful for the development and testing of experiments as well as
functions to join the data output.
"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import sys, os
from importlib import import_module

from . import control, io, show_documentation
from .misc import _secure_hash, get_system_info, download_from_stash


def create_template():
    template_file = '''# -*- coding: utf-8 -*-

"""This file is an automatically created template for an Expyriment experiment.

It has been created by calling `expyriment -C`.

"""


import expyriment as xpy


# SETTINGS
#xpy.control.set_develop_mode()


# DESIGN
exp = xpy.design.Experiment(name="My Experiment")

xpy.control.initialize(exp)


# RUN
xpy.control.start()

xpy.stimuli.TextScreen("Hello World!", "[Press any key to continue]").present()
exp.keyboard.wait()

xpy.control.end(goodbye_text="Thank you for participating!")
'''
    print("Created experiment_template.py")
    f = open('experiment_template.py','w')
    f.write(template_file)
    f.close()

def join_data():
    from .misc import data_preprocessing
    print("Joining data")
    sys.stdout.write(" data subfolder [optional, default=data]? ")
    folder = input()
    if len(folder)<=0:
        folder = "data"
    start_with = input(" data files start with [optional]? ")
    d = data_preprocessing.Aggregator(folder, start_with)
    return d

def main():
    import argparse
    parser = argparse.ArgumentParser(description="""The Expyriment command line interface provides a convenient way to run
experiment scripts and apply default settings, as well as access to a
selection of other common functionality.""",
            epilog="(c) F. Krause & O. Lindemann")

    if len(sys.argv[1:])==0:
        parser.print_help()
        parser.exit()

    parser.add_argument("SCRIPT", action='store', default=None,
                    help="The expyriment script to be executed",
                        nargs='?')

    parser.add_argument("-g", "-0", action="store_true",
                    help="No OpenGL (no vsync / no blocking)")

    parser.add_argument("-1", action="store_true",
                    help="OpenGL (vsync / no blocking)")

    parser.add_argument("-2", action="store_true",
                    help="OpenGL (vsync / blocking)")

    parser.add_argument("-3", action="store_true",
                    help="OpenGL (vsync / alternative blocking)")

    parser.add_argument("-t", action="store_true",
                    help="No time stamps for output files")

    parser.add_argument("-w", action="store_true",
                    help="Window mode")

    parser.add_argument("-f", action="store_true",
                    help="Fast mode (no initialize delay and fast quitting)")

    parser.add_argument("-a", action="store_true",
                    help="Auto create subject ID")

    parser.add_argument("-i", action="store_true",
                    help="Intensive logging (log level 2)")

    parser.add_argument("-d", action="store_true",
                    help="Develop mode (equivalent to -wfat)")

    parser.add_argument("-b", action="store_true",
                    help="Alternative blocking mode (blocking mode 2)")

    parser.add_argument("-C", action="store_true",
                    help="Create experiment template")

    parser.add_argument("-D", action="store_true",
                    help="Download from Expyriment stash")

    parser.add_argument("-J", action="store_true",
                    help="Join data files to one single csv file")

    parser.add_argument("-S", action="store_true",
                    help="Print system information")

    parser.add_argument("-T", action="store_true",
                    help="Run the Expyriment Test Suite")

    parser.add_argument("-A", action="store_true",
                    help="Start the Expyriment API Reference Tool")

    parser.add_argument("-B", action="store_true",
                    help="Open browser with API reference")

    parser.add_argument("--version", action="store_true",
                    help="Print version")

    #parse
    args = vars(parser.parse_args())

    if args['d']:
        control.set_develop_mode(True)
    if args['i']:
        print("* Intensive logging")
        control.defaults.event_logging = 2
    if args['f']:
        print("* Fast mode")
        control.defaults.initialize_delay = 0
        control.defaults.fast_quit = False
    if args['w']:
        print("* Window mode")
        control.defaults.window_mode = True
    if args['g']:
        print("* No OpenGL (no vsync / no blocking)")
        control.defaults.open_gl = False
    if args['1']:
        print("* OpenGL (vsync / no blocking)")
        control.defaults.open_gl = 1
    if args['2']:
        print("* OpenGL (vsync / blocking)")
        control.defaults.open_gl = 2
    if args['3']:
        print("* OpenGL (vsync / alternative blocking)")
        control.defaults.open_gl = 3
    if args['t']:
        print("* No time stamps")
        io.defaults.outputfile_time_stamp = False
    if args['a']:
        print("* Auto create subject id")
        control.defaults.auto_create_subject_id = True
    if args["S"]:
        print("System Info")
        print(get_system_info(as_string=True))
        sys.exit()
    if args["T"]:
        print("Run Test Suite")
        control.run_test_suite()
        sys.exit()
    if args["B"]:
        show_documentation(2)
        sys.exit()
    if args["A"]:
        print("Start API Reference Tool")
        show_documentation(3)
        sys.exit()
    if args["C"]:
        create_template()
        sys.exit()
    if args["D"]:
        print("Download from stash")
        what = ""
        while what not in ["all", "examples", "extras", "tools"]:
            sys.stdout.write(" what to download ([all]/examples/extras/tools)? ")
            what = input()
            if what == "":
                what = "all"
        branches = ["master"]
        if __version__ != "":
            branches.append(__version__)
        branch = ""
        while branch not in branches:
            if len(branches) == 1:
                sys.stdout.write(" from which branch? ([master])? ")
            else:
                sys.stdout.write(" from which branch? (master/[{0}])? ".format(
                branches[1]))
            branch = input()
            if branch == "":
                if len(branches) == 1:
                    branch = "master"
                else:
                    branch = branches[1]
        download_from_stash(what, branch)
    if args["J"]:
        d = join_data()
        output =""
        while len(output) <= 1:
            sys.stdout.write(" name of output csv file? ")
            output = input()
        d.write_concatenated_data(output)
        sys.exit()

    # run script
    if args["SCRIPT"] is not None:
        script = os.path.abspath(args["SCRIPT"])
        if not os.path.isfile(script):
            print("Can't find {0}!".format(args["SCRIPT"]))
            exit()
        path, pyfile = os.path.split(script)
        os.chdir(path)
        sys.argv[0] = script # expyriment expect sys.argv[0] as main filename
        _secure_hash.main_file = script
        secure_hashes = {script : _secure_hash._make_secure_hash(script)}
        secure_hashes = _secure_hash.\
                    _append_hashes_from_imported_modules(secure_hashes, script)
        _secure_hash.secure_hashes = secure_hashes
        _secure_hash.cout_hashes()

        def execfile(filepath, globals=None, locals=None):
            if globals is None:
                globals = {}
            globals.update({
                "__file__": filepath,
                "__name__": "__main__",
            })
            with open(filepath, 'rb') as file:
                exec(compile(file.read(), filepath, 'exec'), globals, locals)
        execfile(pyfile)


if __name__ == "__main__":
    main()



"""
Default settings for the io package.

This module contains default values for all optional arguments in the init
function of all classes in this package.

"""

__author__ = 'Florian Krause <florian@expyriment.org, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

from ..misc import constants as _constants

# Keyboard
keyboard_default_keys = None

#Mouse
mouse_show_cursor = True
mouse_track_button_events = True
mouse_track_motion_events = False

# StreamingButtonBox
streamingbuttonbox_baseline = 0

# OutputFile
outputfile_comment_char = "#"
outputfile_time_stamp = True
outputfile_eol = "\n"

# EventFile
eventfile_directory = "events"
eventfile_delimiter = ","

# DataFile
datafile_directory = "data"
datafile_delimiter = ","

# SeriaPort
serialport_baudrate = 19200
serialport_bytesize = 8
serialport_parity = 'N'
serialport_stopbits = 1
serialport_timeout = 0
serialport_xonxoff = 0
serialport_rtscts = 0
serialport_dsrdtr = 0
serialport_input_history = None
serialport_input_timing = 1
serialport_os_buffer_size = 3000

# MarkerOutput
markeroutput_default_code = 1
markeroutput_default_duration = None

# TriggerInput
triggerinput_default_code = 1

# TextInput
textinput_position = (0, 0)
textinput_character_filter = None
textinput_length = 25
textinput_message_text_size = None  # 'None' is experiment_text_size
textinput_message_colour = None  # 'None is experiment_text_colour
textinput_message_font = None  # 'None' will use default system font
textinput_message_bold = False
textinput_message_italic = False
textinput_message_right_to_left = False
textinput_user_text_size = None  # 'None' is experiment_text_size
textinput_user_text_colour = None  # 'None' is experiment_text_colour
textinput_user_text_font = None  # 'None' will use default system font
textinput_user_text_bold = False
textinput_user_right_to_left = False
textinput_background_colour = None  # 'None' is experiment_background_colour
textinput_frame_colour = None  # 'None' is experiment_text_colour
textinput_gap = 6

# Menu
textmenu_text_size = 20
textmenu_gap = 2
textmenu_position = (0, 0)
textmenu_background_colour = _constants.C_BLACK
textmenu_heading_font = None
textmenu_text_font = None
textmenu_text_colour = _constants.C_GREY
textmenu_heading_text_colour = _constants.C_EXPYRIMENT_ORANGE
textmenu_select_background_colour = _constants.C_DARKGREY
textmenu_select_text_colour = [0, 0, 0]
textmenu_select_frame_colour = _constants.C_EXPYRIMENT_ORANGE
textmenu_select_frame_line_width = 0
textmenu_justification = 1
textmenu_scroll_menu = 0



from __future__ import print_function
import expyriment
from pydoc import getdoc
x = None
y = None


def _get_doc_and_function(obj):
    rtn = []
    for var in dir(obj):
        if not var.startswith("_"):
            rtn.append(var)
    return getdoc(obj), rtn

def _read_module(mod, doc_dict):
    doc_dict[mod.__name__], classes = _get_doc_and_function(mod)
    for cl in classes:
        cl = "{0}.{1}".format(mod.__name__, cl)
        exec("x =" + cl)
        doc_dict[cl], functions = _get_doc_and_function(x)
        for fnc in functions:
            fnc = "{0}.{1}".format(cl, fnc)
            exec("y =" + fnc)
            doc_dict[fnc], _tmp = _get_doc_and_function(y)

def search_doc(search_str, doc_dict):

    for k in doc_dict.keys():
        if k.lower().find(search_str.lower()) > 0 or\
            doc_dict[k].lower().find(search_str.lower()) > 0:
            print("\n-------------------------------------------------------------------------------")
            print("[ {0} ]\n".format(k))
            print("{0}".format(doc_dict[k]))



doc_dict = {}
_read_module(expyriment.io, doc_dict)
_read_module(expyriment.stimuli, doc_dict)
_read_module(expyriment.design, doc_dict)
_read_module(expyriment.misc, doc_dict)

while True:
    search = raw_input("New search (q=quit): ")
    if search == "q":
        break
    else:
        search_doc(search, doc_dict)
        print("\n")




"""
The geometry module.

This module contains miscellaneous geometry functions for expyriment.

"""
from __future__ import absolute_import, print_function, division
from builtins import *

__author__ = 'Florian Krause <florian@expyriment.org>, \
Oliver Lindemann <oliver@expyriment.org>'
__version__ = ''
__revision__ = ''
__date__ = ''

import math as _math
from ... import _internals

def coordinates2position(coordinates, surface_size=None):
    """Convert coordinates on the screen or surface to an
    Expyriment position.

    Parameters
    ----------
    coordinates : (int, int)
        coordinates (x,y) to convert
    surface_size: (int, int), optional
        size of the surface one which position is defined
        if None (default), the position will be calculated relative
        to the screen

    Returns
    -------
    position : [int, int]

    """

    if surface_size is None:
        surface_size = _internals.active_exp.screen.surface.get_size()

    rtn = [coordinates[0] - surface_size[0] // 2,
            - coordinates[1] + surface_size[1] // 2]
    if (surface_size[0] % 2) == 0: #even
        rtn[0] += 1
    if (surface_size[1] % 2) == 0:
        rtn[1] -= 1
    return rtn

def position2coordinate(coordinate, surface_size=None):
    """"OBSOLETE FUNCTION: Please use 'position2coordinates'."""

    raise DeprecationWarning("position2coordinate is an obsolete function. Please use position2coordinates.")

    position2coordinates(coordinate, surface_size=None)

def position2coordinates(position, surface_size=None):
    """Convert an Expyriment position to coordinates on the
    screen or surface.

    Parameters
    ----------
    position : (int, int)
        position (x,y) to convert
    surface_size: (int, int), optional
        size of the surface one which coordinates should be calculated
        if None (default), the coordinats will be calculated relative
        to the screen

    Returns
    -------
    coordinates : [int, int]

    """

    if surface_size is None:
        surface_size = _internals.active_exp.screen.surface.get_size()

    rtn = [position[0] + surface_size[0] // 2,
            - position[1] + surface_size[1] // 2]
    if (surface_size[0] % 2) == 0: #even
        rtn[0] -= 1
    if (surface_size[1] % 2) == 0:
        rtn[1] -= 1
    return rtn

def position2visual_angle(position, viewing_distance, monitor_size):
    """Convert an expyriment position (pixel) to a visual angle from center.

    Parameters
    ----------
    position : (int, int)
        position (x,y) to convert
    viewing_distance : numeric
        viewing distance in cm
    monitior_size : numeric
        physical size of the monitor in cm (x, y)

    Returns
    -------
    angle : (float, float)
        visual angle for x & y dimension

    """

    screen_size = _internals.active_exp.screen.surface.get_size()
    cm = (position[0] * monitor_size[0] / float(screen_size[0]),
          position[1] * monitor_size[1] / float(screen_size[1]))

    angle = (2.0 * _math.atan((cm[0] / 2) / viewing_distance),
             2.0 * _math.atan((cm[1] / 2) / viewing_distance))
    return (angle[0] * 180 / _math.pi, angle[1] * 180 / _math.pi)

def visual_angle2position(visual_angle, viewing_distance, monitor_size):
    """Convert an position defined as visual angle from center to expyriment
    position (pixel).

    Parameters
    ----------
    visual_angle : (numeric, numeric)
        position in visual angle (x,y) to convert
    viewing_distance : numeric
        viewing distance in cm
    monitior_size : (numeric, numeric)
        physical size of the monitor in cm (x, y)

    Returns
    -------
    position : (float, float)
        position (x,y)

    """

    screen_size = _internals.active_exp.screen.surface.get_size()
    angle = (visual_angle[0] * _math.pi / 360,
             visual_angle[1] * _math.pi / 360) # angle / 180 / 2
    cm = (_math.tan(angle[0]) * viewing_distance * 2,
          _math.tan(angle[1]) * viewing_distance * 2)
    return (cm[0] * screen_size[0] / monitor_size[0],
            cm[1] * screen_size[1] / monitor_size[1])

def tuples2points(list_of_tuples):
    """Converts a list of tuples (x,y) to a list of misc.XYPoints"""
    return list(map(lambda v:XYPoint(x=v[0], y=v[1]), list_of_tuples))

def points_to_vertices(points):
    """OBSOLETE FUNCTION! Please use `points2vertices`!"""
    
    raise DeprecationWarning("points_to_vertices is an obsolete function. Please use points2vertices.")

def points2vertices(points):
    """Returns vertex representation of the points (list of misc.XYPoints)

    Parameters
    ----------
    points : (int, int)
        list of points

    Returns
    -------
    vtx : list
        list of vertices

    """

    vtx = []
    for i in range(1, len(points)):
        vtx.append((points[i].x - points[i - 1].x, points[i].y - points[i - 1].y))
    return vtx

def lines_intersect(pa, pb, pc, pd):
    """Returns true if two line segments are intersecting

    Parameters
    ----------
    pa : misc.geometry.XYPoint
        point 1 of line 1
    pb : misc.geometry.XYPoint
        point 2 of line 1
    pc : misc.geometry.XYPoint
        point 1 of line 2
    pb : misc.geometry.XYPoint
        point 2 of line 2

    Returns
    -------
    check : bool
        True if lines intersect

    """

    def ccw(pa, pb, pc):
        return (pc._y - pa._y) * (pb._x - pa._x) > (pb._y - pa._y) * (pc._x - pa._x)

    return ccw(pa, pc, pd) != ccw(pb, pc, pd) and ccw(pa, pb, pc) != ccw(pa, pb, pd)

def lines_intersection_point(pa, pb, pc, pd):
    """Returns the intersection point of two lines (a-b) and (c-d)

    Parameters
    ----------
    pa : misc.geometry.XYPoint
        point 1 of line 1
    pb : misc.geometry.XYPoint
        point 2 of line 1
    pc : misc.geometry.XYPoint
        point 1 of line 2
    pb : misc.geometry.XYPoint
        point 2 of line 2

    Returns
    -------
    intersec_point: misc.geometry.XYPoint
        intersection point

    """

    # slope: dy/dx (y2 - y1) / (x2 - x1)
    slope = ((pb.y - pa.y) / float(pb.x - pa.x), #line 1
             (pd.y - pc.y) / float(pd.x - pc.x)) #line 2
    if slope[0] == slope[1]:
        return None  # lines are parallel

    # intercept: y = slope*x + b,  b = y - slope*x
    intercept =(pa.y - slope[0]*pa.x,
                pc.y - slope[1]*pc.x)

    # Set both lines equal to find the intersection point in the x direction
    # m1 * x + b1 = m2 * x + b2 ==> x = (b2 - b1) / (m1 - m2)
    x = (intercept[1] - intercept[0]) / float(slope[0] - slope[1])
    # solve for y:  y = mx + b
    y = slope[0] * x + intercept[0]
    return XYPoint(x=x,y=y)


def cartesian2polar(xy, radians=False):
    """Convert a cartesian coordinate (x,y) to a polar coordinate
    (radial, angle[degrees]).

    Parameters
    ----------
    xy : (float, float)
        cartesian coordinate (x,y)
    radians : boolean
        use radians instead of degrees for the angle

    Returns
    ----------
    polar : (float, float)
        polar coordinate (radial, angle[degrees])

    """

    ang = _math.atan2(xy[1], xy[0])
    radial =_math.hypot(xy[0], xy[1])
    if radians:
        return (radial, ang)
    else:
        return (radial, _math.degrees(ang))

def polar2cartesian(polar, radians=False):
    """Convert a polar coordinate (radial, angle[degrees]) to a polar
    coordinate (x, y)


    Parameters
    ----------
    polar : (float, float)
        polar coordinate (radial, angle[degrees])
    radians : boolean
        use radians instead of degrees for the angle

    Returns
    ----------
    xy : (float, float)
        cartesian coordinate (x,y)

    """

    if radians:
        a = polar[1]
    else:
        a = _math.radians(polar[1])
    return (polar[0]*_math.cos(a), polar[0]*_math.sin(a))


class XYPoint(object):
    """ The Expyriment point class """
    def __init__(self, x=None, y=None, xy=None):
        """Initialize a XYPoint.

        Parameters
        ----------
        x : numeric
        y : numeric
        xy : (numeric, numeric)
            xy = (x,y)

        Notes
        -----
        use `x`, `y` values (two numberic) or the tuple xy=(x,y)

        """

        if x is None:
            if xy is None:
                self._x = 0
                self._y = 0
            else:
                self._x = xy[0]
                self._y = xy[1]
        elif y is None:
            #if only a tuple is specified: e-g. Point((23,23))
            self._x = x[0]
            self._y = x[1]
        else:
            self._x = x
            self._y = y

    def __repr__(self):
        return  "(x={0}, y={1})".format(self._x, self._y)

    @property
    def x(self):
        """Getter for x"""
        return self._x

    @x.setter
    def x(self, value):
        """Getter for x"""
        self._x = value

    @property
    def y(self):
        """Getter for y"""
        return self._y

    @y.setter
    def y(self, value):
        """Getter for y"""
        self._y = value

    @property
    def tuple(self):
        """Getter for the tuple (x, y) """

        return (self._x, self._y)

    @tuple.setter
    def tuple(self, xy_tuple):
        self._x = xy_tuple[0]
        self._y = xy_tuple[1]


    @property
    def polar(self):
        """Getter for polar coordinate the point """

        return cartesian2polar((self._x, self._y))

    def move(self, v):
        """Move the point along the coordinates specified by the vector v.

        Parameters
        ----------
        v : misc.XYPoint
            movement vector

        """

        self._x = self._x + v._x
        self._y = self._y + v._y
        return self

    def distance(self, p):
        """Return euclidian distance to the points (p).

        Parameters
        ----------
        p : misc.XYPoint
            movement vector

        Returns
        -------
        dist : float
            distance to other point p

        """

        dx = self._x - p._x
        dy = self._y - p._y
        return _math.hypot(dx, dy)

    def rotate(self, degree, rotation_centre=(0, 0)):
        """Rotate the point counterclockwise in degree around rotation_centre.

        Parameters
        ----------
        degree : int
            degree of rotation (default=(0, 0) )
        rotation_center : (numeric, numeric)
            rotation center (x, y)

        """

        #cart -> polar
        r, ang = cartesian2polar(xy = (self._x - rotation_centre[0],
                                       self._y - rotation_centre[1]),
                                 radians=True)
        ang -= _math.radians(degree)
        #polar -> cart
        self._x, self._y = polar2cartesian(polar=(r, ang), radians=True)
        self._x += rotation_centre[0]
        self._y += rotation_centre[1]


    def is_inside_polygon(self, point_list):
        """Return true if point is inside a given polygon.

        Parameters
        ----------
        point_list : list
            point list defining the polygon

        Returns
        -------
        check : bool

"""

        n = len(point_list)
        inside = False

        p1 = point_list[0]
        for i in range(n + 1):
            p2 = point_list[i % n]
            if self._y > min(p1._y, p2._y):
                if self._y <= max(p1._y, p2._y):
                    if self._x <= max(p1._x, p2._x):
                        if p1._y != p2._y:
                            xinters = (self._y - p1._y) * (p2._x - p1._x) // (p2._y - p1._y) + p1._x
                        if p1._x == p2._x or self._x <= xinters:
                            inside = not inside
            p1 = p2

        return inside
